@@ -1,9 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * Author:
- *   Nat Friedman (nat@ximian.com)
+
+/* e-book-backend-file.c - File contact backend.
  *
- * Copyright 2000, Ximian, Inc.
+ * Copyright (C) 2005 Novell, Inc.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Nat Friedman <nat@novell.com>
+ *          Chris Toshok <toshok@ximian.com>
+ *          Hans Petter Jansson <hpj@novell.com>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +66,7 @@ struct _EBookBackendFilePrivate {
 	char     *filename;
 	char     *summary_filename;
 	DB       *file_db;
+	DB_ENV   *env;
 	EBookBackendSummary *summary;
 };
 
@@ -58,6 +76,7 @@ string_to_dbt(const char *str, DBT *dbt)
 	memset (dbt, 0, sizeof (*dbt));
 	dbt->data = (void*)str;
 	dbt->size = strlen (str) + 1;
+	dbt->flags = DB_DBT_USERMEM;
 }
 
 static EContact*
@@ -157,6 +176,7 @@ do_create(EBookBackendFile  *bf,
 static EBookBackendSyncStatus
 e_book_backend_file_create_contact (EBookBackendSync *backend,
 				    EDataBook *book,
+				    guint32 opid,
 				    const char *vcard,
 				    EContact **contact)
 {
@@ -176,7 +196,8 @@ e_book_backend_file_create_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_remove_contacts (EBookBackendSync *backend,
-				     EDataBook    *book,
+				     EDataBook *book,
+				     guint32 opid,
 				     GList *id_list,
 				     GList **ids)
 {
@@ -222,7 +243,8 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_modify_contact (EBookBackendSync *backend,
-				    EDataBook    *book,
+				    EDataBook *book,
+				    guint32 opid,
 				    const char *vcard,
 				    EContact **contact)
 {
@@ -235,6 +257,9 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	*contact = e_contact_new_from_vcard (vcard);
 	id = e_contact_get(*contact, E_CONTACT_UID);
 
+	if (id == NULL)
+		return GNOME_Evolution_Addressbook_OtherError;
+
 	/* This is disgusting, but for a time cards were added with
            ID's that are no longer used (they contained both the uri
            and the id.) If we recognize it as a uri (file:///...) trim
@@ -248,12 +273,15 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 	string_to_dbt (lookup_id, &id_dbt);	
 	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+	vcard_dbt.flags = DB_DBT_MALLOC;
 
 	/* get the old ecard - the one that's presently in the db */
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 	if (0 != db_error)
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 
+	free (vcard_dbt.data);
+
 	string_to_dbt (vcard, &vcard_dbt);	
 
 	db_error = db->put (db, NULL, &id_dbt, &vcard_dbt, 0);
@@ -276,7 +304,8 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_contact (EBookBackendSync *backend,
-				 EDataBook    *book,
+				 EDataBook *book,
+				 guint32 opid,
 				 const char *id,
 				 char **vcard)
 {
@@ -290,11 +319,13 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 
 	string_to_dbt (id, &id_dbt);
 	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+	vcard_dbt.flags = DB_DBT_MALLOC;
 
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 	if (db_error == 0) {
 		*vcard = g_strdup (vcard_dbt.data);
+		free (vcard_dbt.data);
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
 		*vcard = g_strdup ("");
@@ -304,7 +335,8 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_contact_list (EBookBackendSync *backend,
-				      EDataBook    *book,
+				      EDataBook *book,
+				      guint32 opid,
 				      const char *query,
 				      GList **contacts)
 {
@@ -317,50 +349,75 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 	gboolean search_needed;
 	const char *search = query;
 	GList *contact_list = NULL;
-
+	EBookBackendSyncStatus status;
+	
 	d(printf ("e_book_backend_file_get_contact_list (%s)\n", search));
+	status = GNOME_Evolution_Addressbook_Success;
+	if (e_book_backend_summary_is_summary_query (bf->priv->summary, search)) {
+	
+		/* do a summary query */
+		GPtrArray *ids = e_book_backend_summary_search (bf->priv->summary, search);
+		int i;
 
-	search_needed = TRUE;
+		for (i = 0; i < ids->len; i ++) {
+			char *id = g_ptr_array_index (ids, i);
+			string_to_dbt (id, &id_dbt);
+			memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+			vcard_dbt.flags = DB_DBT_MALLOC;
 
-	if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
-		search_needed = FALSE;
-
-	card_sexp = e_book_backend_sexp_new (search);
-	if (!card_sexp) {
-		/* XXX this needs to be an invalid query error of some sort*/
-		return GNOME_Evolution_Addressbook_ContactNotFound;
-	}
-
-	db_error = db->cursor (db, NULL, &dbc, 0);
-
-	if (db_error != 0) {
-		/* XXX this needs to be some CouldNotOpen error */
-		return GNOME_Evolution_Addressbook_ContactNotFound;
-	}
-
-	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
-	memset (&id_dbt, 0, sizeof (id_dbt));
-	db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
-
-	while (db_error == 0) {
-
-		/* don't include the version in the list of cards */
-		if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
-		    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
-
-			if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
+			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
+			if (db_error == 0)
 				contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+			else { 
+				status = GNOME_Evolution_Addressbook_OtherError ;
+				break;
 			}
 		}
+		g_ptr_array_free (ids, TRUE);
+		
+	} else {
+		search_needed = TRUE;
+		if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
+			search_needed = FALSE;
 
-		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+		card_sexp = e_book_backend_sexp_new (search);
+		if (!card_sexp) {
+			/* XXX this needs to be an invalid query error of some sort*/
+			return GNOME_Evolution_Addressbook_ContactNotFound;
+		}
 
+		db_error = db->cursor (db, NULL, &dbc, 0);
+
+		if (db_error != 0) {
+			/* XXX this needs to be some CouldNotOpen error */
+			return GNOME_Evolution_Addressbook_ContactNotFound;
+		}
+
+		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+		memset (&id_dbt, 0, sizeof (id_dbt));
+		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+
+		while (db_error == 0) {
+
+			/* don't include the version in the list of cards */
+			if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
+			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
+
+				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
+					contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+				}
+			}
+
+			db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+
+		}
+		status = db_error != DB_NOTFOUND
+			? GNOME_Evolution_Addressbook_OtherError
+			: GNOME_Evolution_Addressbook_Success;
 	}
 
 	*contacts = contact_list;
-	return db_error != DB_NOTFOUND
-		? GNOME_Evolution_Addressbook_OtherError
-		: GNOME_Evolution_Addressbook_Success;
+	return status;
 }
 
 typedef struct {
@@ -451,7 +508,8 @@ book_view_thread (gpointer data)
 
 			string_to_dbt (id, &id_dbt);
 			memset (&vcard_dbt, 0, sizeof (vcard_dbt));
-				
+			vcard_dbt.flags = DB_DBT_MALLOC;
+
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 			if (db_error == 0) {
@@ -459,6 +517,11 @@ book_view_thread (gpointer data)
 				/* notify_update will check if it matches for us */
 				e_data_book_view_notify_update (book_view, contact);
 				g_object_unref (contact);
+
+				free (vcard_dbt.data);
+			}
+			else {
+				g_warning ("db->get returned %d", db_error);
 			}
 		}
 
@@ -569,6 +632,8 @@ e_book_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 	
 	string_to_dbt (key, &id_dbt);
 	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+	vcard_dbt.flags = DB_DBT_MALLOC;
+
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 	
 	if (db_error != 0) {
@@ -587,12 +652,15 @@ e_book_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 						vcard_string);
 
 		g_object_unref (contact);
+		
+		free (vcard_dbt.data);
 	}
 }
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_changes (EBookBackendSync *backend,
-				 EDataBook    *book,
+				 EDataBook *book,
+				 guint32 opid,
 				 const char *change_id,
 				 GList **changes_out)
 {
@@ -733,7 +801,8 @@ e_book_backend_file_extract_path_from_uri (const char *uri)
 
 static EBookBackendSyncStatus
 e_book_backend_file_authenticate_user (EBookBackendSync *backend,
-				       EDataBook    *book,
+				       EDataBook *book,
+				       guint32 opid,
 				       const char *user,
 				       const char *passwd,
 				       const char *auth_method)
@@ -741,9 +810,25 @@ e_book_backend_file_authenticate_user (EBookBackendSync *backend,
 	return GNOME_Evolution_Addressbook_Success;
 }
 
+static EBookBackendSyncStatus
+e_book_backend_file_get_required_fields (EBookBackendSync *backend,
+					  EDataBook *book,
+					  guint32 opid,
+					  GList **fields_out)
+{
+	GList *fields = NULL;
+	int i;
+
+	fields = g_list_append (fields , g_strdup(e_contact_field_name (E_CONTACT_FILE_AS)));
+	*fields_out = fields;
+	return GNOME_Evolution_Addressbook_Success;
+}
+
+
 static EBookBackendSyncStatus
 e_book_backend_file_get_supported_fields (EBookBackendSync *backend,
-					  EDataBook    *book,
+					  EDataBook *book,
+					  guint32 opid,
 					  GList **fields_out)
 {
 	GList *fields = NULL;
@@ -865,11 +950,13 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 
 	string_to_dbt (E_BOOK_BACKEND_FILE_VERSION_NAME, &version_name_dbt);
 	memset (&version_dbt, 0, sizeof (version_dbt));
+	version_dbt.flags = DB_DBT_MALLOC;
 
 	db_error = db->get (db, NULL, &version_name_dbt, &version_dbt, 0);
 	if (db_error == 0) {
 		/* success */
 		version = g_strdup (version_dbt.data);
+		free (version_dbt.data);
 	}
 	else {
 		/* key was not in file */
@@ -884,7 +971,16 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 	return ret_val;
 }
 
-#include <libedata-book/ximian-vcard.h>
+#ifdef CREATE_DEFAULT_VCARD
+# include <libedata-book/ximian-vcard.h>
+#endif
+
+static void
+file_errcall (const char *buf1, char *buf2)
+{
+	g_warning ("libdb error: %s", buf2);
+}
+
 
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_file_load_source (EBookBackend           *backend,
@@ -896,6 +992,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	gboolean        writable = FALSE;
 	int             db_error;
 	DB *db;
+	DB_ENV *env;
 	time_t db_mtime;
 	struct stat sb;
 	gchar *uri;
@@ -912,13 +1009,22 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
-	db_error = db_create (&db, NULL, 0);
+	db_error = db_env_create (&env, 0);
+	if (db_error != 0) {
+		g_warning ("db_env_create failed with %d", db_error);
+		return GNOME_Evolution_Addressbook_OtherError;
+	}
+	env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
+
+	env->set_errcall (env, file_errcall);
+
+	db_error = db_create (&db, env, 0);
 	if (db_error != 0) {
 		g_warning ("db_create failed with %d", db_error);
 		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
-	db_error = db->open (db, NULL, filename, NULL, DB_HASH, 0, 0666);
+	db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
 
 	if (db_error == DB_OLD_VERSION) {
 		db_error = e_db3_utils_upgrade_format (filename);
@@ -928,7 +1034,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
-		db_error = db->open (db, NULL, filename, NULL, DB_HASH, 0, 0666);
+		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
 	}
 
 	bf->priv->file_db = db;
@@ -936,7 +1042,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	if (db_error == 0) {
 		writable = TRUE;
 	} else {
-		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_RDONLY, 0666);
+		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_RDONLY | DB_THREAD, 0666);
 
 		if (db_error != 0 && !only_if_exists) {
 			int rv;
@@ -952,16 +1058,18 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 					return GNOME_Evolution_Addressbook_OtherError;
 			}
 
-			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE, 0666);
+			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
 			if (db_error != 0) {
 				g_warning ("db->open (... DB_CREATE ...) failed with %d", db_error);
 			}
 			else {
+#ifdef CREATE_DEFAULT_VCARD
 				EContact *contact;
 
 				contact = do_create(bf, XIMIAN_VCARD);
 				/* XXX check errors here */
 				g_object_unref (contact);
+#endif
 
 				writable = TRUE;
 			}
@@ -1004,7 +1112,6 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	e_book_backend_set_is_loaded (backend, TRUE);
 	e_book_backend_set_is_writable (backend, writable);
-
 	return GNOME_Evolution_Addressbook_Success;
 }
 
@@ -1028,7 +1135,8 @@ select_changes (const char *name)
 
 static EBookBackendSyncStatus
 e_book_backend_file_remove (EBookBackendSync *backend,
-			    EDataBook        *book)
+			    EDataBook        *book,
+			    guint32           opid)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	GDir *dir;
@@ -1077,7 +1185,7 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 static char *
 e_book_backend_file_get_static_capabilities (EBookBackend *backend)
 {
-	return g_strdup("local,do-initial-query,bulk-removes");
+	return g_strdup("local,do-initial-query,bulk-removes,contact-lists");
 }
 
 static GNOME_Evolution_Addressbook_CallStatus
@@ -1085,7 +1193,14 @@ e_book_backend_file_cancel_operation (EBookBackend *backend, EDataBook *book)
 {
 	return GNOME_Evolution_Addressbook_CouldNotCancel;
 }
-
+static void 
+e_book_backend_file_set_mode (EBookBackend *backend,  GNOME_Evolution_Addressbook_BookMode mode)
+{
+	if (e_book_backend_is_loaded (backend)) {
+		e_book_backend_notify_writable (backend, TRUE);
+		e_book_backend_notify_connection_status (backend, TRUE);
+	}
+}
 static gboolean
 e_book_backend_file_construct (EBookBackendFile *backend)
 {
@@ -1125,6 +1240,12 @@ e_book_backend_file_dispose (GObject *object)
 	bf = E_BOOK_BACKEND_FILE (object);
 
 	if (bf->priv) {
+		if (bf->priv->file_db)
+			bf->priv->file_db->close (bf->priv->file_db, 0);
+
+		if (bf->priv->env)
+			bf->priv->env->close (bf->priv->env, 0);
+
 		if (bf->priv->summary)
 			g_object_unref(bf->priv->summary);
 		g_free (bf->priv->filename);
@@ -1156,7 +1277,7 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	backend_class->start_book_view         = e_book_backend_file_start_book_view;
 	backend_class->stop_book_view          = e_book_backend_file_stop_book_view;
 	backend_class->cancel_operation        = e_book_backend_file_cancel_operation;
-
+	backend_class->set_mode                = e_book_backend_file_set_mode;
 	sync_class->remove_sync                = e_book_backend_file_remove;
 	sync_class->create_contact_sync        = e_book_backend_file_create_contact;
 	sync_class->remove_contacts_sync       = e_book_backend_file_remove_contacts;
@@ -1166,6 +1287,8 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	sync_class->get_changes_sync           = e_book_backend_file_get_changes;
 	sync_class->authenticate_user_sync     = e_book_backend_file_authenticate_user;
 	sync_class->get_supported_fields_sync  = e_book_backend_file_get_supported_fields;
+	sync_class->get_required_fields_sync   = e_book_backend_file_get_required_fields;
+	
 
 	object_class->dispose = e_book_backend_file_dispose;
 }
