@@ -3,8 +3,8 @@
  * Authors : 
  *  JP Rosevear <jpr@ximian.com>
  *  Rodrigo Moya <rodrigo@ximian.com>
- *
- * Copyright 2003, Novell, Inc.
+ *  Harish Krishnaswamy <kharish@novell.com>
+ *  Copyright 2003, Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of version 2 of the GNU General Public 
@@ -21,12 +21,23 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <e-gw-connection.h>
 #include <e-gw-message.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-time-util.h>
+#include <libsoup/soup-misc.h>
 #include "e-cal-backend-groupwise-utils.h"
+#include <libedataserver/e-source-list.h>
 
 static gboolean 
 get_recur_instance (ECalComponent *comp, time_t instance_start, time_t instance_end, gpointer data)
@@ -116,6 +127,211 @@ set_categories_for_gw_item (EGwItem *item, GList *category_names, ECalBackendGro
              e_gw_item_set_categories (item, category_ids);
 }
 
+static void
+add_send_options_data_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default_zone)
+{
+	const char *x_val;
+	const char *x_name;
+	icalcomponent *icalcomp;
+	icalproperty *icalprop;
+	struct icaltimetype temp;
+	gboolean sendoptions_set = FALSE;
+
+	icalcomp = e_cal_component_get_icalcomponent (comp);
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
+
+	while (icalprop) {
+
+		x_name = icalproperty_get_x_name (icalprop);
+		
+		if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-PRIORITY")) {
+			sendoptions_set = TRUE;
+			x_val = icalproperty_get_x (icalprop);
+			switch (atoi (x_val)) {
+				case 1:  e_gw_item_set_priority (item, E_GW_ITEM_PRIORITY_HIGH);
+					 break;
+				case 2:  e_gw_item_set_priority (item, E_GW_ITEM_PRIORITY_STANDARD);
+					 break;
+				case 3:	 e_gw_item_set_priority (item, E_GW_ITEM_PRIORITY_LOW);
+					 break;
+				default: e_gw_item_set_priority (item, NULL);
+					 break;
+			}
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-REPLY")) {
+			e_gw_item_set_reply_request (item, TRUE);
+			x_val = icalproperty_get_x (icalprop);
+			if (strcmp (x_val, "convenient"))
+				e_gw_item_set_reply_within (item, (char *) x_val);	
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-EXPIRE")) {
+			char *expire = NULL;
+			x_val = icalproperty_get_x (icalprop);
+			temp = icaltime_current_time_with_zone (default_zone ? default_zone : icaltimezone_get_utc_timezone ());
+			icaltime_adjust (&temp, atoi (x_val), 0, 0, 0); 
+			icaltime_set_timezone (&temp, default_zone);
+			temp = icaltime_convert_to_zone (temp, icaltimezone_get_utc_timezone ());
+			expire = icaltime_as_ical_string (temp);
+			e_gw_item_set_expires (item, (char *) expire);
+			g_free (expire);
+
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-DELAY")) {
+			char *delay = NULL;
+			x_val = icalproperty_get_x (icalprop);
+			temp = icaltime_from_string (x_val);
+			icaltime_set_timezone (&temp, default_zone);
+			temp = icaltime_convert_to_zone (temp, icaltimezone_get_utc_timezone ());
+			delay = icaltime_as_ical_string (temp);
+			e_gw_item_set_delay_until (item, (char *) delay);
+			g_free (delay);
+					
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-TRACKINFO")) {
+			sendoptions_set = TRUE;
+			x_val = icalproperty_get_x (icalprop);
+			switch (atoi (x_val)) {
+				case 1: e_gw_item_set_track_info (item, E_GW_ITEM_DELIVERED);
+					break;
+				case 2:	e_gw_item_set_track_info (item, E_GW_ITEM_DELIVERED_OPENED);
+					break;
+				case 3: e_gw_item_set_track_info (item, E_GW_ITEM_ALL);
+					break;
+				default: e_gw_item_set_track_info (item, E_GW_ITEM_NONE);
+					 break;
+			}
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-OPENED")) {
+			int i = 0;
+			x_val = icalproperty_get_x (icalprop);
+			i = atoi (x_val);
+			switch (i) {
+				case 0: e_gw_item_set_notify_opened (item, E_GW_ITEM_NOTIFY_NONE);
+					break;
+				case 1: e_gw_item_set_notify_opened (item, E_GW_ITEM_NOTIFY_MAIL);
+			}
+			
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-ACCEPTED")) {
+			int i = 0;
+			x_val = icalproperty_get_x (icalprop);
+			i = atoi (x_val);
+			switch (i) {
+				case 0: e_gw_item_set_notify_accepted (item, E_GW_ITEM_NOTIFY_NONE);
+					break;
+				case 1: e_gw_item_set_notify_accepted (item, E_GW_ITEM_NOTIFY_MAIL);
+			}
+
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-DECLINED")) {
+			int i = 0;
+			x_val = icalproperty_get_x (icalprop);
+			i = atoi (x_val);
+			switch (i) {
+				case 0: e_gw_item_set_notify_declined (item, E_GW_ITEM_NOTIFY_NONE);
+					break;
+				case 1: e_gw_item_set_notify_declined (item, E_GW_ITEM_NOTIFY_MAIL);
+			}
+
+		} else if (!strcmp (x_name, "X-EVOLUTION-OPTIONS-COMPLETED")) {
+			int i = 0;
+			x_val = icalproperty_get_x (icalprop);
+			i = atoi (x_val);
+			switch (i) {
+				case 0: e_gw_item_set_notify_completed (item, E_GW_ITEM_NOTIFY_NONE);
+					break;
+				case 1: e_gw_item_set_notify_completed (item, E_GW_ITEM_NOTIFY_MAIL);
+			}
+		}
+
+		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
+	}
+
+	e_gw_item_set_sendoptions (item, sendoptions_set);
+
+}
+
+static void
+e_cal_backend_groupwise_set_attachments_from_comp (ECalComponent *comp,
+		EGwItem *item)
+{
+	GSList *attach_list = NULL, *attach_file_list = NULL;
+	GSList *l;
+
+	e_cal_component_get_attachment_list (comp, &attach_file_list);
+	
+	for (l = attach_file_list; l ; l = l->next) {
+		
+		EGwItemAttachment *attach_item;
+		char *file_contents, *encoded_data;
+		int fd, len;
+		int len_read = 0;
+		char buf[1024];
+		struct stat sb;
+		char *attach_filename_full, *filename;
+		const char *uid;
+		
+
+		attach_filename_full = (char *)l->data + 7;
+		attach_item = g_new0 (EGwItemAttachment, 1);
+		/* FIXME the member does not follow the naming convention.
+		 * Should be fixed in e-gw-item*/
+		attach_item->contentType = g_strdup (gnome_vfs_get_mime_type (attach_filename_full));
+
+		/*
+		 * Would gnome_vfs_async be better suited for this ?
+		 */
+		fd = open (attach_filename_full, O_RDONLY); 
+		if (fd == -1) {
+			/* TODO handle error conditions */
+			g_free (attach_item);
+			g_message ("DEBUG: could not open the file descriptor\n");
+		}
+		if (fstat (fd, &sb) == -1) {
+			/* TODO handle error conditions */
+			g_free (attach_item);
+			g_message ("DEBUG: could not fstat the attachment file\n");
+			continue;
+		}
+		len = sb.st_size;
+
+		file_contents = g_malloc (len + 1);
+	
+		while (len_read < len) {
+			int c = read (fd, buf, sizeof (buf));
+
+			if (c == -1)
+				break;
+
+			memcpy (&file_contents[len_read], buf, c);
+			len_read += c;
+		}
+		file_contents [len_read] = 0;
+
+		/* Extract the simple file name from the
+		 * attach_filename_full which is of the form
+		 * file://<path>/compuid-<simple filename> 
+		 */
+		e_cal_component_get_uid (comp, &uid);
+		filename = g_strrstr (attach_filename_full, uid); 		
+
+		if (filename == NULL) {
+			/* TODO handle error conditions */
+			g_free (attach_item);
+			g_message ("DEBUG:\n This is an invalid attachment file\n");
+			continue;
+		}	
+
+
+		attach_item->name = g_strdup (filename + strlen(uid) + 1);
+		/* do a base64 encoding so it can be embedded in a soap
+		 * message */
+		/* TODO handle error conditions */
+		encoded_data = soup_base64_encode (file_contents, len_read);
+		attach_item->data = encoded_data;
+		attach_item->size = strlen (encoded_data); 
+
+		g_free (file_contents);
+		close (fd);
+		attach_list = g_slist_append (attach_list, attach_item);
+	}
+
+	e_gw_item_set_attach_id_list (item, attach_list);
+}
+
 static EGwItem *
 set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBackendGroupwise *cbgw)
 {
@@ -127,13 +343,12 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 	int *priority;
 	GList *categories;
 	GSList *slist, *sl;
-	icaltimezone *default_zone;
+	icaltimezone *default_zone, *utc;
 	struct icaltimetype itt_utc;
 	
 	default_zone = e_cal_backend_groupwise_get_default_zone (cbgw);
+	utc = icaltimezone_get_utc_timezone ();
 
-	g_return_if_fail (default_zone != NULL);
-	
 	/* first set specific properties */
 	switch (e_cal_component_get_vtype (comp)) {
 	case E_CAL_COMPONENT_EVENT :
@@ -173,8 +388,8 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 		e_cal_component_get_dtend (comp, &dt);
 		if (dt.value) {
 			if (!icaltime_get_timezone (*dt.value))
-				icaltime_set_timezone (dt.value, default_zone);
-			itt_utc = icaltime_convert_to_zone (*dt.value, icaltimezone_get_utc_timezone ());
+				icaltime_set_timezone (dt.value, default_zone ? default_zone : utc);
+			itt_utc = icaltime_convert_to_zone (*dt.value, utc);
 			e_gw_item_set_end_date (item, icaltime_as_ical_string (itt_utc));
 		}
 
@@ -188,27 +403,27 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 		if (dt.value) {
 			if (!icaltime_get_timezone (*dt.value))
 				icaltime_set_timezone (dt.value, default_zone);
-			itt_utc = icaltime_convert_to_zone (*dt.value, icaltimezone_get_utc_timezone ());
+			itt_utc = icaltime_convert_to_zone (*dt.value, utc);
 			e_gw_item_set_due_date (item, icaltime_as_ical_string (itt_utc));
 		}
+		
+			/* priority */
+		 priority = NULL;
+		 e_cal_component_get_priority (comp, &priority);
+		 if (priority && *priority) {
+			 if (*priority >= 7)
+				 e_gw_item_set_task_priority (item, E_GW_ITEM_PRIORITY_LOW);
+			 else if (*priority >= 5)
+				 e_gw_item_set_task_priority (item, E_GW_ITEM_PRIORITY_STANDARD);
+			 else if (*priority >= 1)
+				 e_gw_item_set_task_priority (item, E_GW_ITEM_PRIORITY_HIGH);
+			 else
+				 e_gw_item_set_task_priority (item, NULL);
 
-		/* priority */
-		priority = NULL;
-		e_cal_component_get_priority (comp, &priority);
-		if (priority && *priority) {
-			if (*priority >= 7)
-				e_gw_item_set_priority (item, E_GW_ITEM_PRIORITY_LOW);
-			else if (*priority >= 5)
-				e_gw_item_set_priority (item, E_GW_ITEM_PRIORITY_STANDARD);
-			else if (*priority >= 1)
-				e_gw_item_set_priority (item, E_GW_ITEM_PRIORITY_HIGH);
-			else
-				e_gw_item_set_priority (item, NULL);
-
-			e_cal_component_free_priority (priority);
-		}
-
-		/* completed */
+			 e_cal_component_free_priority (priority);
+		 }
+	  
+			/* completed */
 		e_cal_component_get_completed (comp, &dt.value);
 		if (dt.value) {
 			e_gw_item_set_completed (item, TRUE);
@@ -253,12 +468,13 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 		e_cal_component_free_text_list (slist);
 	}
 
+
 	/* start date */
 	e_cal_component_get_dtstart (comp, &dt);
 	if (dt.value) {
 		if (!icaltime_get_timezone (*dt.value))
 			icaltime_set_timezone (dt.value, default_zone);
-		itt_utc = icaltime_convert_to_zone (*dt.value, icaltimezone_get_utc_timezone ());
+		itt_utc = icaltime_convert_to_zone (*dt.value, utc);
 		e_gw_item_set_start_date (item, icaltime_as_ical_string (itt_utc));
 	} else if (e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_APPOINTMENT) {
 		/* appointments need the start date property */
@@ -275,7 +491,7 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 	if (dt.value) {
 		if (!icaltime_get_timezone (*dt.value))
 			icaltime_set_timezone (dt.value, default_zone);
-		itt_utc = icaltime_convert_to_zone (*dt.value, icaltimezone_get_utc_timezone ());
+		itt_utc = icaltime_convert_to_zone (*dt.value, utc); 
 		e_gw_item_set_creation_date (item, icaltime_as_ical_string (itt_utc));
 	} else {
 		struct icaltimetype itt;
@@ -331,6 +547,10 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 		}
 				
 		e_gw_item_set_recipient_list (item, recipient_list);
+
+		/* Send Options */
+		add_send_options_data_to_item (item, comp, default_zone);
+
 	}
 
 	if (e_cal_component_has_organizer (comp)) {
@@ -353,12 +573,17 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 		if (dt.tzid)
 			e_cal_recur_generate_instances (comp, -1, -1,get_recur_instance, &recur_dates, resolve_tzid_cb, NULL, (icaltimezone *) default_zone);		
 		else 
-			e_cal_recur_generate_instances (comp, -1, -1,get_recur_instance, &recur_dates, resolve_tzid_cb, NULL, icaltimezone_get_utc_timezone());		
+			e_cal_recur_generate_instances (comp, -1, -1,get_recur_instance, &recur_dates, resolve_tzid_cb, NULL, utc);		
 
 		recur_dates = g_slist_delete_link (recur_dates, recur_dates);
 		
 		e_gw_item_set_recurrence_dates (item, recur_dates);
 	    }
+	
+	/* attachments */
+	if (e_cal_component_has_attachments (comp)) {
+		e_cal_backend_groupwise_set_attachments_from_comp (comp, item); 
+	}
 
 	return item;
 }
@@ -376,13 +601,97 @@ e_gw_item_new_from_cal_component (const char *container, ECalBackendGroupwise *c
 	return set_properties_from_cal_component (item, comp, cbgw);
 }
 
+/* Fetch data from the server and unencode it to the actual data 
+ * and populate the attach_data
+ */
+static gboolean
+get_attach_data_from_server (GSList *attach_item_list, ECalBackendGroupwise *cbgw)
+{
+	GSList *l;
+	EGwConnection *cnc;
+	EGwConnectionStatus status;
+
+	cnc = e_cal_backend_groupwise_get_connection (cbgw);
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
+
+	for (l = attach_item_list; l; l = l->next) {
+		EGwItemAttachment *attach_item = (EGwItemAttachment *) l->data;
+	
+		status = e_gw_connection_get_attachment (cnc, attach_item->id, 0, -1, 
+			(const char *)&(attach_item->data), &(attach_item->size)); 
+
+		if (status != E_GW_CONNECTION_STATUS_OK ) {
+			g_warning ("Failed to read the attachment from the server\n");
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
+static void
+set_attachments_to_cal_component (EGwItem *item, ECalComponent *comp, ECalBackendGroupwise *cbgw)
+{
+	/*TODO if attach_list just has attach_ids - fetch the
+	 * attachments. For each, serialize them in the attach
+	 * store location and set the url in the cal component*/
+	GSList *attach_data = NULL, *attach_id_list, *l;
+	GSList *comp_attachment_list = NULL;
+
+	
+	attach_id_list = e_gw_item_get_attach_id_list (item);
+	if (attach_id_list == NULL)
+		return;
+	
+	/* Download the data and populate the structure */
+	if (!get_attach_data_from_server (attach_id_list, cbgw))
+		return;
+
+	
+	/* FIXME the code below assumes the attach_data now contains
+	 * EGwItemAttachment structures and proceeds to serialize 
+	 * them. Check base64 encoding assumptions by mailer code*/
+
+	for (l = attach_data; l ; l = l->next) {
+		int fd, length;
+		char *attach_file_url, *attach_data;
+		const char *uid;
+		EGwItemAttachment *attach_item;
+
+		attach_item = (EGwItemAttachment *) l->data;
+		/*TODO decode the encoded data on attachments */
+		if (attach_item->size > 0) {
+			attach_data = soup_base64_decode (attach_item->data, &length);
+		}
+		e_cal_component_get_uid (comp, &uid);
+		attach_file_url = g_strconcat
+			(e_cal_backend_groupwise_get_local_attachments_store (cbgw), 
+			 "/", uid, "-", attach_item->name, NULL);
+		fd = open (attach_file_url+7, O_RDWR|O_CREAT|O_TRUNC, 0600);
+		if (fd == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not serialize attachments\n");
+		}
+
+		if (write (fd, attach_data, length) == -1) {
+			/* TODO handle error condition */
+			g_message ("DEBUG: attachment write failed.\n");
+		}
+		
+		g_free (attach_data);
+		close (fd);
+
+		comp_attachment_list = g_slist_append
+			(comp_attachment_list, attach_file_url);
+	}
+	
+}
 ECalComponent *
 e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 {
 	ECalComponent *comp;
 	ECalComponentText text;
 	ECalComponentDateTime dt;
-	const char *description;
+	const char *description, *uid;
 	char *t, *name;
 	GList *category_ids, *categories;
 	GHashTable *categories_by_id;
@@ -396,6 +705,7 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 	GSList *recipient_list, *rl, *attendee_list = NULL;
 	EGwItemOrganizer *organizer;
 	EGwItemType item_type;
+	GSList *attach_list;
 
 	default_zone = e_cal_backend_groupwise_get_default_zone (cbgw);
 	categories_by_id = e_cal_backend_groupwise_get_categories_by_id (cbgw);
@@ -427,8 +737,13 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 	}
 
 	/* UID */
-	e_cal_component_set_uid (comp, e_gw_item_get_icalid (item));
-
+	uid = e_gw_item_get_icalid (item);
+	if (uid)
+		e_cal_component_set_uid (comp, e_gw_item_get_icalid (item));
+	else {
+		g_object_unref (comp);
+		return NULL;
+	}
 	/* summary */
 	text.value = e_gw_item_get_subject (item);
 	text.altrep = NULL;
@@ -566,6 +881,33 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		e_cal_component_set_organizer (comp, cal_organizer);
 	}
 
+	/* set attachments */
+	attach_list = e_gw_item_get_attach_id_list (item);	
+	if (attach_list != NULL) {
+		/* Iterate thro the list and stuff it into ical*/
+		EGwItemAttachment *attach_data;
+		int attach_length;
+		unsigned char *attachment;
+		EGwConnectionStatus status;
+
+	 	/* First check if the item has attachments . if yes check if it 
+		 * exists on the filesystem or pull it from the server,
+		 * serialize it.
+		 */
+		/*  nice if we could just get it from the cache/item not
+		 * from a n/w call here */
+		attach_data = (EGwItemAttachment *) attach_list->data;
+		status  = e_gw_connection_get_attachment (e_cal_backend_groupwise_get_connection (cbgw), attach_data->id, 0, 0, 
+					(const char *)&attachment, &attach_length); 
+		if (status == E_GW_CONNECTION_STATUS_OK) 
+			g_message ("DEBUG : Spewing out the attachment\n%s\n", attachment);		
+		else 
+			g_message ("DEBUG:attachment call was a disaster\n");
+
+		set_attachments_to_cal_component (item, comp, cbgw);
+	
+	}
+
 	/* set specific properties */
 	switch (item_type) {
 	case E_GW_ITEM_TYPE_APPOINTMENT :
@@ -641,7 +983,7 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			e_cal_component_set_due (comp, &dt);
 		}
 		/* priority */
-		description = e_gw_item_get_priority (item);
+		description = e_gw_item_get_task_priority (item);
 		if (description) {
 			if (!strcmp (description, E_GW_ITEM_PRIORITY_STANDARD))
 				priority = 5;
@@ -657,6 +999,7 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		/* EGwItem's completed is a boolean */
 		if (e_gw_item_get_completed (item)) {
 			percent = 100;
+			
 			t = e_gw_item_get_completed_date (item);
 			if (t) {
 				itt_utc = icaltime_from_string (t);
@@ -684,17 +1027,22 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		return NULL;
 	}
 
+	e_cal_component_commit_sequence (comp);
+	g_message ("DEBUG:\n %s\n", e_cal_component_get_as_string (comp));
+
 	return comp;
 }
 
 EGwConnectionStatus
-e_gw_connection_send_appointment (EGwConnection *cnc, const char *container, ECalComponent *comp, icalproperty_method method, gboolean *remove)
+e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *container, ECalComponent *comp, icalproperty_method method, gboolean *remove, ECalComponent **created_comp)
 {
+	EGwConnection *cnc;
 	EGwConnectionStatus status;
 	icalparameter_partstat partstat;
 	char *item_id;
 	
 
+	cnc = e_cal_backend_groupwise_get_connection (cbgw);
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 	g_return_val_if_fail (E_IS_CAL_COMPONENT (comp), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
@@ -782,12 +1130,19 @@ e_gw_connection_send_appointment (EGwConnection *cnc, const char *container, ECa
 		*remove = TRUE;
 		break;
 	default:
-		status = E_GW_CONNECTION_STATUS_INVALID_OBJECT;
-	}	
+		return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
+	}
 	
 	if (status == E_GW_CONNECTION_STATUS_ITEM_ALREADY_ACCEPTED)
 		return E_GW_CONNECTION_STATUS_OK;
 
+	if (!*remove && status == E_GW_CONNECTION_STATUS_OK) {
+		EGwItem *item;
+
+		status = e_gw_connection_get_item (cnc, container, item_id, "recipients message recipientStatus attachments default", &item);
+		*created_comp = e_gw_item_to_cal_component (item, cbgw);
+	}
+
 	return status;
 }
 
@@ -796,38 +1151,13 @@ e_gw_connection_create_appointment (EGwConnection *cnc, const char *container, E
 {
 	EGwItem *item;
 	EGwConnectionStatus status;
-	icalproperty *icalprop;
-	gboolean move_cal = FALSE;
-	icalcomponent *icalcomp;
-	char *id = NULL;
 
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 	g_return_val_if_fail (E_IS_CAL_COMPONENT (comp), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
-	icalcomp = e_cal_component_get_icalcomponent (comp);
-	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
-	while (icalprop) {
-		const char *x_name;
-
-		x_name = icalproperty_get_x_name (icalprop);
-		if (!strcmp (x_name, "X-EVOLUTION-MOVE-CALENDAR")) {
-			move_cal = TRUE;
-			break;
-		}
-
-		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
-	}
-
 	item = e_gw_item_new_from_cal_component (container, cbgw, comp);
 	e_gw_item_set_container_id (item, container);
-	if (!move_cal)
-		status = e_gw_connection_send_item (cnc, item, id_list);
-	else {
-		e_gw_item_set_source (item, "personal");
-		status = e_gw_connection_create_item (cnc, item, &id);
-		*id_list = g_slist_append (*id_list, id);
-	}
-
+	status = e_gw_connection_send_item (cnc, item, id_list);
 	g_object_unref (item);
 
 	return status;
@@ -945,6 +1275,8 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
         EGwConnectionStatus status;
         SoupSoapParameter *param, *subparam;
         const char *session;
+	gboolean resend_request = TRUE;
+	int request_iteration = 0;
 
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 
@@ -954,6 +1286,9 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
         if (status != E_GW_CONNECTION_STATUS_OK)
                 return status;
 
+	resend :
+	while (resend_request) {
+
         /* getFreeBusy */
         /* build the SOAP message */
 	msg = e_gw_message_new_with_header (e_gw_connection_get_uri (cnc),
@@ -984,6 +1319,29 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 
+	for (subparam = soup_soap_parameter_get_first_child_by_name (param, "freeBusyStats");
+	     subparam != NULL;
+	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "freeBusyStats")) {
+		SoupSoapParameter *param_outstanding;
+		const char *outstanding;
+
+		param_outstanding = soup_soap_parameter_get_first_child_by_name (subparam, "outstanding");
+		if (param_outstanding)
+			outstanding = soup_soap_parameter_get_string_value (param_outstanding);
+		/* Try 12 times - this is approximately 2 minutes of time to
+		 * obtain the free/busy information from the server */
+		if (strcmp (outstanding, "0") && (request_iteration < 12)) {
+			request_iteration++;
+			g_object_unref (msg);
+		        g_object_unref (response);
+			g_usleep (10000000);
+			goto resend;
+		}
+
+	}
+
+	resend_request = FALSE;
+
         for (subparam = soup_soap_parameter_get_first_child_by_name (param, "user");
 	     subparam != NULL;
 	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "user")) {
@@ -1081,13 +1439,15 @@ e_gw_connection_get_freebusy_info (EGwConnection *cnc, GList *users, time_t star
 		}
 
 		e_cal_component_commit_sequence (comp);
-		*freebusy = g_list_append (*freebusy, g_strdup (e_cal_component_get_as_string (comp)));
+		*freebusy = g_list_append (*freebusy, e_cal_component_get_as_string (comp));
 		g_object_unref (comp);
 	}
 
         g_object_unref (msg);
         g_object_unref (response);
 
+	} /* end of while loop */
+
         /* closeFreeBusySession*/
         return close_freebusy_session (cnc, session);
 }
@@ -1154,7 +1514,7 @@ e_gw_item_set_changes (EGwItem *item, EGwItem *cache_item)
 	const char *classification, *cache_classification;
 	const char *accept_level, *cache_accept_level;
 	const char *place, *cache_place;
-	const char *priority, *cache_priority;
+	const char *task_priority, *cache_task_priority;
 	int trigger, cache_trigger;
 	char *due_date, *cache_due_date;
 	char *start_date, *cache_start_date;
@@ -1195,6 +1555,130 @@ e_gw_item_set_changes (EGwItem *item, EGwItem *cache_item)
 	}
 	else if ( e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_TASK) {
 		SET_DELTA(due_date);
-		SET_DELTA (priority);
+		SET_DELTA(task_priority);
 	}
 }
+
+
+static void 
+add_return_value (EGwSendOptionsReturnNotify track, ESource *source, char *notify)
+{
+	char *value;
+	
+	switch (track) {
+		case E_GW_RETURN_NOTIFY_MAIL:
+			value =  g_strdup ("mail");
+			break;
+		default:
+			value = g_strdup ("none");		
+	}
+	
+	e_source_set_property (source, notify, value);
+	g_free (value), value = NULL;
+}
+
+void
+e_cal_backend_groupwise_store_settings (EGwSendOptions *opts, ECalBackendGroupwise *cbgw)
+{
+	EGwSendOptionsGeneral *gopts;
+	EGwSendOptionsStatusTracking *sopts;
+	icaltimetype tt;
+	icalcomponent_kind kind;
+	GConfClient *gconf = gconf_client_get_default ();
+	ESource *source;
+	ESourceList *source_list;
+	const char *uid;
+	char *value;
+
+	source = e_cal_backend_get_source (E_CAL_BACKEND (cbgw));
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw)); 
+
+	gopts = e_gw_sendoptions_get_general_options (opts);
+	if (kind == ICAL_VEVENT_COMPONENT) {
+		sopts = e_gw_sendoptions_get_status_tracking_options (opts, "calendar");
+		source_list = e_source_list_new_for_gconf (gconf, "/apps/evolution/calendar/sources");
+	} else {
+		source_list = e_source_list_new_for_gconf (gconf, "/apps/evolution/tasks/sources");
+		sopts = e_gw_sendoptions_get_status_tracking_options (opts, "task");
+	}
+
+	uid = e_source_peek_uid (source);
+	source = e_source_list_peek_source_by_uid (source_list, uid);
+	if (gopts) {
+			/* priority */
+		switch (gopts->priority) {
+			case E_GW_PRIORITY_HIGH:
+				value = g_strdup ("high");
+				break;
+			case E_GW_PRIORITY_STANDARD:
+				value = g_strdup ("standard");
+				break;
+			case E_GW_PRIORITY_LOW:
+				value =  g_strdup ("low");
+				break;
+			default:
+				value = g_strdup ("undefined");
+		}
+		e_source_set_property (source, "priority", value);
+		g_free (value), value = NULL;
+
+			/* Reply Requested */
+		/*TODO Fill the value if it is not "convinient" */
+		if (gopts->reply_enabled) {
+			if (gopts->reply_convenient)
+				value = g_strdup ("convinient");
+			else 
+				value = g_strdup_printf ("%d",gopts->reply_within);
+		 } else
+			value = g_strdup ("none");
+		e_source_set_property (source, "reply-requested", value);
+		g_free (value), value = NULL;
+		
+			/* Delay delivery */
+		if (gopts->delay_enabled) {
+				tt = icaltime_today ();
+				icaltime_adjust (&tt, gopts->delay_until, 0, 0, 0);
+				value = icaltime_as_ical_string (tt);
+		} else
+			value = g_strdup ("none");
+		e_source_set_property (source, "delay-delivery", value);
+		g_free (value), value = NULL;
+		
+			/* Expiration date */
+		if (gopts->expiration_enabled)
+			value =  g_strdup_printf ("%d", gopts->expire_after);
+		else
+			value = g_strdup ("none");
+		e_source_set_property (source, "expiration", value);
+		g_free (value), value = NULL;
+	}
+		
+	if (sopts) {
+			/* status tracking */
+		if (sopts->tracking_enabled) {
+			switch (sopts->track_when) {
+				case E_GW_DELIVERED :
+					value = g_strdup ("delivered");
+					break;
+				case E_GW_DELIVERED_OPENED:
+					value = g_strdup ("delivered-opened");
+					break;
+				default:
+					value = g_strdup ("all");
+			}
+		} else
+			value = g_strdup ("none");
+		e_source_set_property (source, "status-tracking", value);
+		g_free (value), value = NULL;
+
+		add_return_value (sopts->opened, source, "return-open"); 
+		add_return_value (sopts->accepted, source, "return-accept"); 
+		add_return_value (sopts->declined, source, "return-decline"); 
+		add_return_value (sopts->completed, source, "return-complete"); 
+	}	
+
+	g_object_unref (gconf);
+}
+
+
+	
