@@ -26,6 +26,10 @@
 #endif
 
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <bonobo/bonobo-i18n.h>
@@ -58,6 +62,7 @@ struct _ECalBackendGroupwisePrivate {
 
 	/* fields for storing info while offline */
 	char *user_email;
+	char *local_attachments_store;
 };
 
 static void e_cal_backend_groupwise_dispose (GObject *object);
@@ -132,7 +137,9 @@ populate_cache (ECalBackendGroupwise *cbgw)
 		g_mutex_unlock (mutex);
                 return status;
         }
-	status = e_gw_connection_create_cursor (priv->cnc, priv->container_id, "recipients message recipientStatus default", NULL, &cursor);
+	status = e_gw_connection_create_cursor (priv->cnc,
+			priv->container_id, 
+			"recipients message recipientStatus attachments default", NULL, &cursor);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
 		g_mutex_unlock (mutex);
@@ -204,13 +211,15 @@ get_deltas (gpointer handle)
         
 	if (!handle)
 		return FALSE;
-	
 	cbgw = (ECalBackendGroupwise *) handle;
 	priv= cbgw->priv;
  	cnc = priv->cnc; 
  	cache = priv->cache; 
 	item_list = NULL;
 	
+	if (priv->mode == CAL_MODE_LOCAL)
+		return FALSE;
+
 	if (!mod_time) {
 		cache_file_name = e_file_cache_get_filename (E_FILE_CACHE (priv->cache));
 		printf ("%s %d\n", cache_file_name, stat (cache_file_name, &buf));
@@ -225,6 +234,8 @@ get_deltas (gpointer handle)
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
 		return TRUE;
 	}
+
+	e_file_cache_freeze_changes (E_FILE_CACHE (cache));
 	for (; item_list != NULL; item_list = g_slist_next(item_list)) {
 		EGwItem *item = E_GW_ITEM(item_list->data);
 		ECalComponent *comp = e_gw_item_to_cal_component (item, cbgw);
@@ -250,13 +261,16 @@ get_deltas (gpointer handle)
 		g_slist_free (item_list);
 		item_list = NULL;
 	}
+	e_file_cache_thaw_changes (E_FILE_CACHE (cache));
+
 	status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id,"recipients message recipientStatus  default", time_string, "Modified", "CalendarItem", NULL,  -1,  &item_list);
 	
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
 		return TRUE;
 	}
-	
+
+	e_file_cache_freeze_changes (E_FILE_CACHE (cache));
 	for (; item_list != NULL; item_list = g_slist_next(item_list)) {
 		EGwItem *item = E_GW_ITEM(item_list->data);
 		ECalComponent *modified_comp, *cache_comp;
@@ -279,6 +293,8 @@ get_deltas (gpointer handle)
 		g_object_unref (item);
 		g_object_unref (modified_comp);
 	}
+	e_file_cache_thaw_changes (E_FILE_CACHE (cache));
+
 	if (item_list) {
 		g_slist_free (item_list);
 		item_list = NULL;
@@ -301,7 +317,8 @@ get_deltas (gpointer handle)
 				g_slist_find_custom (cache_keys, l->data, (GCompareFunc) strcmp));
 		g_free (l->data);
 	}
-	
+
+	e_file_cache_freeze_changes (E_FILE_CACHE (cache));
 	for (l = cache_keys; l ; l = g_slist_next (l)) {
 		/* assumes rid is null - which works for now */
 		ECalComponent *comp = NULL;
@@ -313,12 +330,14 @@ get_deltas (gpointer handle)
 				(vtype == E_CAL_COMPONENT_TODO)) {
 			comp_str = e_cal_component_get_as_string (comp);
 			e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), 
-								(char *) l->data, comp_str);
+							     (char *) l->data, comp_str, NULL);
 			e_cal_backend_cache_remove_component (cache, (const char *) l->data, NULL);
 			g_free (comp_str);
 		}
 		g_object_unref (comp);
 	}
+	e_file_cache_thaw_changes (E_FILE_CACHE (cache));
+
 	if (item_list) {
 		g_slist_free (item_list);
 		item_list = NULL;
@@ -357,7 +376,7 @@ form_uri (ESource *source)
 		port = "7181";
 	use_ssl = e_source_get_property (source, "use_ssl");
 
-	if (use_ssl)
+	if (use_ssl && !g_str_equal (use_ssl, "never"))
 		formed_uri = g_strconcat ("https://", parsed_uri->host,":", port, "/soap", NULL );
 	else 
 		formed_uri = g_strconcat ("http://", parsed_uri->host,":", port, "/soap", NULL );
@@ -367,13 +386,14 @@ form_uri (ESource *source)
 	return formed_uri;
 
 }
-
+				
 static ECalBackendSyncStatus
 cache_init (ECalBackendGroupwise *cbgw)
 {
 	ECalBackendGroupwisePrivate *priv = cbgw->priv;
 	EGwConnectionStatus cnc_status;
 	icalcomponent_kind kind;
+	EGwSendOptions *opts;
 	const char *time_interval_string;
 	int time_interval;
 
@@ -386,6 +406,11 @@ cache_init (ECalBackendGroupwise *cbgw)
 		time_interval *= (60*1000); 
 		
 	}
+	cnc_status = e_gw_connection_get_settings (priv->cnc, &opts);	
+	if (cnc_status == E_GW_CONNECTION_STATUS_OK) {
+		e_cal_backend_groupwise_store_settings (opts, cbgw);
+		g_object_unref (opts);
+	}
 	
 	/* We poke the cache for a default timezone. Its
 	 * absence indicates that the cache file has not been
@@ -478,6 +503,17 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 
 		if (priv->cnc && priv->cache) {
 			priv->mode = CAL_MODE_REMOTE;
+			
+			if (!priv->timeout_id) {
+				if (get_deltas (cbgw)) {
+					priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				} else {
+					g_warning (G_STRLOC ": Could not populate the cache");
+	ECalBackendSyncStatus status;
+					return GNOME_Evolution_Calendar_PermissionDenied;	
+				}
+			}	 
+
 			return GNOME_Evolution_Calendar_Success;
 		}
 
@@ -599,9 +635,16 @@ e_cal_backend_groupwise_finalize (GObject *object)
 		priv->user_email = NULL;
 	}
 
-	if (priv->timeout_id) 
+	if (priv->local_attachments_store) {
+		g_free (priv->local_attachments_store);
+		priv->local_attachments_store = NULL;
+	}
+
+	if (priv->timeout_id) {
 		g_source_remove (priv->timeout_id);
-	
+		priv->timeout_id = 0;
+	}
+
 	g_free (priv);
 	cbgw->priv = NULL;
 
@@ -676,12 +719,32 @@ e_cal_backend_groupwise_get_static_capabilities (ECalBackendSync *backend, EData
 				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE "," \
 				  CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK "," \
 				  CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR "," \
+				  CAL_STATIC_CAPABILITY_REQ_SEND_OPTIONS "," \
 				  CAL_STATIC_CAPABILITY_SAVE_SCHEDULES);
 
 	return GNOME_Evolution_Calendar_Success;
 }
 
 
+static void
+in_offline (ECalBackendGroupwise *cbgw) {
+	ECalBackendGroupwisePrivate *priv;
+
+	priv= cbgw->priv;
+	priv->read_only = TRUE;
+
+	if (priv->timeout_id) { 
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id =0;
+	}	
+	
+	if (priv->cnc) {
+		g_object_unref (priv->cnc);
+		priv->cnc = NULL;
+	}
+
+	
+}
 
 /* Open handler for the file backend */
 static ECalBackendSyncStatus
@@ -691,6 +754,8 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
 	ECalBackendSyncStatus status;
+	char *mangled_uri;
+	int i;
 	
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
@@ -698,15 +763,57 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 	g_mutex_lock (priv->mutex);
 
 	cbgw->priv->read_only = FALSE;
-	priv->mode = CAL_MODE_LOCAL;
+
+	if (priv->mode == CAL_MODE_LOCAL) {
+		ESource *source;
+		const char *display_contents = NULL;
+			
+		cbgw->priv->read_only = TRUE;				
+		source = e_cal_backend_get_source (E_CAL_BACKEND (cbgw));
+		display_contents = e_source_get_property (source, "offline_sync");
+		
+		if (!display_contents || !g_str_equal (display_contents, "1")) {
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Repository is Offline"));
+			return GNOME_Evolution_Calendar_RepositoryOffline;
+		}
+
+		if (!priv->cache) {
+			priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)));
+			if (!priv->cache) {
+				g_mutex_unlock (priv->mutex);
+				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
+				
+				return GNOME_Evolution_Calendar_OtherError;
+			}
+		}
+		
+		g_mutex_unlock (priv->mutex);	
+		return GNOME_Evolution_Calendar_Success;
+	}
+
 	priv->username = g_strdup (username);
 	priv->password = g_strdup (password);
-	
+
+	/* Set the local attachment store*/
+	mangled_uri = g_strdup (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)));
+	/* mangle the URI to not contain invalid characters */
+	for (i = 0; i < strlen (mangled_uri); i++) {
+		switch (mangled_uri[i]) {
+		case ':' :
+		case '/' :
+			mangled_uri[i] = '_';
+		}
+	}
+
+	priv->local_attachments_store = 
+		g_strconcat ("file://", g_get_home_dir (), "/", ".evolution/cache/calendar",
+			     "/", mangled_uri, NULL);
+	g_free (mangled_uri);
+
 	/* FIXME: no need to set it online here when we implement the online/offline stuff correctly */
 	status = connect_to_server (cbgw);
 
 	g_mutex_unlock (priv->mutex);
-
 	return status;
 }
 
@@ -760,7 +867,6 @@ e_cal_backend_groupwise_get_mode (ECalBackend *backend)
 static void
 e_cal_backend_groupwise_set_mode (ECalBackend *backend, CalMode mode)
 {
-	ECalBackendSyncStatus status;
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
 
@@ -777,24 +883,21 @@ e_cal_backend_groupwise_set_mode (ECalBackend *backend, CalMode mode)
 
 	switch (mode) {
 	case CAL_MODE_REMOTE :/* go online */
-		status = connect_to_server (cbgw);
-		if (status == GNOME_Evolution_Calendar_Success)
-			e_cal_backend_notify_mode (backend,
-						   GNOME_Evolution_Calendar_CalListener_MODE_SET,
-						   GNOME_Evolution_Calendar_MODE_REMOTE);
-		else
-			e_cal_backend_notify_mode (backend,
-						   GNOME_Evolution_Calendar_CalListener_MODE_NOT_SET,
-						   GNOME_Evolution_Calendar_MODE_REMOTE);
+		priv->mode = CAL_MODE_REMOTE;
+		priv->read_only = FALSE;
+		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
+						    GNOME_Evolution_Calendar_MODE_REMOTE);
+		if(e_cal_backend_groupwise_is_loaded (backend))
+		              e_cal_backend_notify_auth_required(backend);	
 		break;
+
 	case CAL_MODE_LOCAL : /* go offline */
 		/* FIXME: make sure we update the cache before closing the connection */
-		g_object_unref (priv->cnc);
-		priv->cnc = NULL;
 		priv->mode = CAL_MODE_LOCAL;
-
+		in_offline (cbgw);
 		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
 					   GNOME_Evolution_Calendar_MODE_LOCAL);
+
 		break;
 	default :
 		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
@@ -1040,6 +1143,11 @@ e_cal_backend_groupwise_get_free_busy (ECalBackendSync *backend, EDataCal *cal,
        cbgw = E_CAL_BACKEND_GROUPWISE (backend);
        cnc = cbgw->priv->cnc;
 
+       if (cbgw->priv->mode == CAL_MODE_LOCAL) {
+	       in_offline (cbgw);
+	       return GNOME_Evolution_Calendar_RepositoryOffline;
+       }
+
        status = e_gw_connection_get_freebusy_info (cnc, users, start, end, freebusy, cbgw->priv->default_zone);
        if (status != E_GW_CONNECTION_STATUS_OK)
                return GNOME_Evolution_Calendar_OtherError;
@@ -1231,6 +1339,11 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 	g_return_val_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL && *calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
 
+	if (priv->mode == CAL_MODE_LOCAL) {
+		in_offline(cbgw);
+		return GNOME_Evolution_Calendar_RepositoryOffline;
+	}
+
 	/* check the component for validity */
 	icalcomp = icalparser_parse_string (*calobj);
 	if (!icalcomp)
@@ -1275,7 +1388,8 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 			}
 			
 			/* convert uid_list to GPtrArray and get the items in a list */
-			e_gw_connection_get_items_from_ids (priv->cnc, priv->container_id, "recipients message",
+			e_gw_connection_get_items_from_ids (priv->cnc,
+					priv->container_id, "recipients message default",
 					uid_array, &list);
 			/* FIXME  check if list is null and status may have
 			 * failed. */
@@ -1318,7 +1432,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal, const char *calobj, 
-				       CalObjModType mod, char **old_object)
+				       CalObjModType mod, char **old_object, char **new_object)
 {
 	ECalBackendGroupwise *cbgw;
         ECalBackendGroupwisePrivate *priv;
@@ -1334,6 +1448,11 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 	g_return_val_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
 
+	if (priv->mode == CAL_MODE_LOCAL) {
+		in_offline (cbgw);
+		return GNOME_Evolution_Calendar_RepositoryOffline;
+	}
+
 	/* check the component for validity */
 	icalcomp = icalparser_parse_string (calobj);
 	if (!icalcomp)
@@ -1402,7 +1521,8 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 				       const char *uid, const char *rid,
-				       CalObjModType mod, char **object)
+				       CalObjModType mod, char **old_object,
+				       char **object)
 {
 	ECalBackendGroupwise *cbgw;
         ECalBackendGroupwisePrivate *priv;
@@ -1411,6 +1531,8 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
+	*old_object = *object = NULL;
+
 	/* if online, remove the item from the server */
 	if (priv->mode == CAL_MODE_REMOTE) {
 		ECalBackendSyncStatus status;
@@ -1422,6 +1544,8 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		if (status != GNOME_Evolution_Calendar_Success)
 			return status;
 
+		*old_object = calobj;
+
 		icalcomp = icalparser_parse_string (calobj);
 		if (!icalcomp) {
 			g_free (calobj);
@@ -1465,6 +1589,9 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 			g_free (calobj);
 			return GNOME_Evolution_Calendar_OtherError;
 		}
+	} else if (priv->mode == CAL_MODE_LOCAL) {
+		in_offline (cbgw);
+		return GNOME_Evolution_Calendar_RepositoryOffline;
 	}
 
 	/* remove the component from the cache */
@@ -1477,6 +1604,80 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	return GNOME_Evolution_Calendar_Success;
 }
 
+static void
+fetch_attachments (ECalBackendGroupwise *cbgw, ECalComponent *comp)
+{
+	GSList *attach_list = NULL, *new_attach_list = NULL;
+	GSList *l;
+	char  *attach_store, *filename, *file_contents;
+	char *dest_url, *dest_file;
+	int fd, len;
+	int len_read = 0;
+	char buf[1024];
+	struct stat sb;
+	const char *uid;
+
+
+	e_cal_component_get_attachment_list (comp, &attach_list);
+	e_cal_component_get_uid (comp, &uid);
+	/*FIXME  get the uri rather than computing the path */
+	attach_store = g_strconcat
+			(e_cal_backend_groupwise_get_local_attachments_store (cbgw), NULL);
+	
+	for (l = attach_list; l ; l = l->next) {
+		char *sfname = (char *)l->data;
+
+		filename = g_strrstr (sfname, "/") + 1;	
+
+		// open the file using the data
+		fd = open (sfname, O_RDONLY); 
+		if (fd == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not open the file descriptor\n");
+			continue;
+		}
+		if (fstat (fd, &sb) == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not fstat the attachment file\n");
+			continue;
+		}
+		len = sb.st_size;
+
+		file_contents = g_malloc (len + 1);
+	
+		while (len_read < len) {
+			int c = read (fd, buf, sizeof (buf));
+
+			if (c == -1)
+				break;
+
+			memcpy (&file_contents[len_read], buf, c);
+			len_read += c;
+		}
+		file_contents [len_read] = 0;
+
+		/* write*/
+		dest_file = g_strconcat (attach_store, "/", uid, "-",
+				filename, NULL);
+		fd = open (dest_file, O_RDWR|O_CREAT|O_TRUNC, 0600);
+		if (fd == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not serialize attachments\n");
+		}
+
+		if (write (fd, file_contents, len_read) == -1) {
+			/* TODO handle error condition */
+			g_message ("DEBUG: attachment write failed.\n");
+		}
+
+		dest_url = g_strconcat ("file:///", dest_file, NULL);
+		new_attach_list = g_slist_append (new_attach_list, dest_url);
+		g_free (dest_file);
+	}
+	g_free (attach_store);
+	e_cal_component_set_attachment_list (comp, new_attach_list);
+}
+
 static ECalBackendSyncStatus
 receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalcomp)
 {
@@ -1492,10 +1693,15 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp));
 	method = icalcomponent_get_method (icalcomp);
 	
+	/* handle attachments */
+	if (e_cal_component_has_attachments (comp))
+		fetch_attachments (cbgw, comp);
 	status = e_gw_connection_send_appointment (cbgw, priv->container_id, comp, method, &remove, &modif_comp);
 
-	if (status == E_GW_CONNECTION_STATUS_OK && !modif_comp)
+	if (status == E_GW_CONNECTION_STATUS_OK && !modif_comp) {
+		g_object_unref (comp);
 		return GNOME_Evolution_Calendar_Success;
+	}
 
 	/* update the cache */
 	if (status == E_GW_CONNECTION_STATUS_OK) {
@@ -1504,11 +1710,11 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 			
 			e_cal_component_get_uid (comp, (const char **) &uid);
 			e_cal_backend_cache_remove_component (priv->cache, uid, NULL);
-			e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), uid, e_cal_component_get_as_string (comp));
+			e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), uid, e_cal_component_get_as_string (comp), NULL);
 			g_free (comp);
 		}
 		else {
-			char *cache_comp = NULL, *temp;
+			char *cache_comp = NULL, *temp, *new_comp = NULL;
 			ECalComponent *cache_component;
 			
 			e_cal_component_commit_sequence (modif_comp);
@@ -1521,23 +1727,26 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 			}
 
 			e_cal_backend_cache_put_component (priv->cache, modif_comp);	
-			
+			e_cal_component_commit_sequence (modif_comp);
+			new_comp = e_cal_component_get_as_string (modif_comp);
+
 			if (cache_comp)
-				e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp, e_cal_component_get_as_string (modif_comp));
+				e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp, new_comp);
 			else
-				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), e_cal_component_get_as_string (modif_comp));
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), new_comp);
 				
 			g_free (cache_comp);
-			g_free (modif_comp);
+			g_free (new_comp);
 			g_free (temp);
-			g_free (cache_component);
 		}
-		
+		g_object_unref (comp);	
 		return GNOME_Evolution_Calendar_Success;
 	}
 
-	if (status == E_GW_CONNECTION_STATUS_INVALID_OBJECT)
+	if (status == E_GW_CONNECTION_STATUS_INVALID_OBJECT) {
+		g_object_unref (comp);
 		return  GNOME_Evolution_Calendar_InvalidObject;
+	}
 	return GNOME_Evolution_Calendar_OtherError;
 }
 
@@ -1554,6 +1763,11 @@ e_cal_backend_groupwise_receive_objects (ECalBackendSync *backend, EDataCal *cal
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
+	if (priv->mode == CAL_MODE_LOCAL) {
+		in_offline (cbgw);
+		return GNOME_Evolution_Calendar_RepositoryOffline;
+	}
+
 	icalcomp = icalparser_parse_string (calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
@@ -1602,7 +1816,7 @@ send_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalcomp,
 		if (found_comp) {
 			char *comp_str;
 			status = e_cal_backend_groupwise_modify_object (E_CAL_BACKEND_SYNC (cbgw), cal, comp_str,
-									CALOBJ_MOD_THIS, &comp_str);
+									CALOBJ_MOD_THIS, &comp_str, NULL);
 			g_free (comp_str);
 		} else
 			status = e_cal_backend_groupwise_create_object (E_CAL_BACKEND_SYNC (cbgw), cal, &comp_str, NULL);
@@ -1638,6 +1852,11 @@ e_cal_backend_groupwise_send_objects (ECalBackendSync *backend, EDataCal *cal, c
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
+	if (priv->mode == CAL_MODE_LOCAL) {
+		in_offline (cbgw);
+		return GNOME_Evolution_Calendar_RepositoryOffline;
+	}
+
 	icalcomp = icalparser_parse_string (calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
@@ -1693,6 +1912,7 @@ e_cal_backend_groupwise_init (ECalBackendGroupwise *cbgw, ECalBackendGroupwiseCl
 	priv->categories_by_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->categories_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->timeout_id = 0;
+	priv->cnc = NULL;
 
 	/* create the mutex for thread safety */
 	priv->mutex = g_mutex_new ();
@@ -1700,7 +1920,7 @@ e_cal_backend_groupwise_init (ECalBackendGroupwise *cbgw, ECalBackendGroupwiseCl
 	cbgw->priv = priv;
 }
 
-/* Class initialization function for the file backend */
+/* Class initialization function for the gw backend */
 static void
 e_cal_backend_groupwise_class_init (ECalBackendGroupwiseClass *class)
 {
@@ -1791,3 +2011,10 @@ e_cal_backend_groupwise_notify_error_code (ECalBackendGroupwise *cbgw, EGwConnec
 	if (msg)
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), msg);
 }
+
+const char *
+e_cal_backend_groupwise_get_local_attachments_store (ECalBackendGroupwise *cbgw)
+{
+	g_return_val_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), NULL);
+	return cbgw->priv->local_attachments_store;
+}
