@@ -45,13 +45,14 @@
 #include "camel-imap4-folder.h"
 #include "camel-imap4-stream.h"
 #include "camel-imap4-command.h"
+#include "camel-imap4-journal.h"
 #include "camel-imap4-utils.h"
 
 #include "camel-imap4-summary.h"
 
 #define d(x) x
 
-#define IMAP4_SUMMARY_VERSION  1
+#define CAMEL_IMAP4_SUMMARY_VERSION  1
 
 static void camel_imap4_summary_class_init (CamelIMAP4SummaryClass *klass);
 static void camel_imap4_summary_init (CamelIMAP4Summary *summary, CamelIMAP4SummaryClass *klass);
@@ -62,7 +63,9 @@ static int imap4_header_save (CamelFolderSummary *summary, FILE *fout);
 static CamelMessageInfo *imap4_message_info_new_from_header (CamelFolderSummary *summary, struct _camel_header_raw *header);
 static CamelMessageInfo *imap4_message_info_load (CamelFolderSummary *summary, FILE *fin);
 static int imap4_message_info_save (CamelFolderSummary *summary, FILE *fout, CamelMessageInfo *info);
-
+static CamelMessageInfo *imap4_message_info_clone (CamelFolderSummary *summary, const CamelMessageInfo *mi);
+static CamelMessageContentInfo *imap4_content_info_load (CamelFolderSummary *summary, FILE *in);
+static int imap4_content_info_save (CamelFolderSummary *summary, FILE *out, CamelMessageContentInfo *info);
 
 static CamelFolderSummaryClass *parent_class = NULL;
 
@@ -99,6 +102,9 @@ camel_imap4_summary_class_init (CamelIMAP4SummaryClass *klass)
 	summary_class->message_info_new_from_header = imap4_message_info_new_from_header;
 	summary_class->message_info_load = imap4_message_info_load;
 	summary_class->message_info_save = imap4_message_info_save;
+	summary_class->message_info_clone = imap4_message_info_clone;
+	summary_class->content_info_load = imap4_content_info_load;
+	summary_class->content_info_save = imap4_content_info_save;
 }
 
 static void
@@ -106,11 +112,11 @@ camel_imap4_summary_init (CamelIMAP4Summary *summary, CamelIMAP4SummaryClass *kl
 {
 	CamelFolderSummary *folder_summary = (CamelFolderSummary *) summary;
 	
-	folder_summary->version += IMAP4_SUMMARY_VERSION;
 	folder_summary->flags = CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_DELETED |
 		CAMEL_MESSAGE_DRAFT | CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN;
 	
 	folder_summary->message_info_size = sizeof (CamelIMAP4MessageInfo);
+	folder_summary->content_info_size = sizeof (CamelIMAP4MessageContentInfo);
 	
 	summary->update_flags = TRUE;
 	summary->uidvalidity_changed = FALSE;
@@ -142,9 +148,18 @@ imap4_header_load (CamelFolderSummary *summary, FILE *fin)
 	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_load (summary, fin) == -1)
 		return -1;
 	
-	if (camel_file_util_decode_uint32 (fin, &imap4_summary->uidvalidity) == -1)
+	if (camel_file_util_decode_fixed_int32 (fin, &imap4_summary->version) == -1)
 		return -1;
 	
+	if (camel_file_util_decode_fixed_int32 (fin, &imap4_summary->uidvalidity) == -1)
+		return -1;
+	
+	if (imap4_summary->version > CAMEL_IMAP4_SUMMARY_VERSION) {
+		g_warning ("Unknown IMAP4 summary version\n");
+		errno = EINVAL;
+		return -1;
+	}
+	
 	return 0;
 }
 
@@ -156,7 +171,10 @@ imap4_header_save (CamelFolderSummary *summary, FILE *fout)
 	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_save (summary, fout) == -1)
 		return -1;
 	
-	if (camel_file_util_encode_uint32 (fout, imap4_summary->uidvalidity) == -1)
+	if (camel_file_util_encode_fixed_int32 (fout, CAMEL_IMAP4_SUMMARY_VERSION) == -1)
+		return -1;
+	
+	if (camel_file_util_encode_fixed_int32 (fout, imap4_summary->uidvalidity) == -1)
 		return -1;
 	
 	return 0;
@@ -939,11 +957,78 @@ untagged_fetch_all (CamelIMAP4Engine *engine, CamelIMAP4Command *ic, guint32 ind
 					g_assert_not_reached ();
 				}
 			} else {
-				g_free(info->uid);
+				g_free (info->uid);
 				info->uid = g_strdup (uid);
 				g_hash_table_insert (fetch->uid_hash, (void *) camel_message_info_uid (info), envelope);
 				changed |= IMAP4_FETCH_UID;
 			}
+		} else if (!strcmp (token->v.atom, "BODY[HEADER.FIELDS")) {
+			/* Mailing-List headers... */
+			struct _camel_header_raw *h;
+			CamelMimeParser *parser;
+			unsigned char *literal;
+			size_t n;
+			
+			/* '(' */
+			if (camel_imap4_engine_next_token (engine, token, ex) == -1)
+				goto exception;
+			
+			if (token->token != '(')
+				goto unexpected;
+			
+			/* Mailing-List header name list */
+			do {
+				if (camel_imap4_engine_next_token (engine, token, ex) == -1)
+					goto exception;
+				
+				if (token->token == ')')
+					break;
+				
+				switch (token->token) {
+				case CAMEL_IMAP4_TOKEN_ATOM:
+				case CAMEL_IMAP4_TOKEN_QSTRING:
+					break;
+				case CAMEL_IMAP4_TOKEN_LITERAL:
+					if (camel_imap4_engine_literal (engine, &literal, &n, ex) == -1)
+						return -1;
+					
+					g_free (literal);
+					break;
+				default:
+					goto unexpected;
+				}
+				
+				/* we don't care what the list was... */
+			} while (1);
+			
+			/* ']' */
+			if (camel_imap4_engine_next_token (engine, token, ex) == -1)
+				goto exception;
+			
+			if (token->token != ']')
+				goto unexpected;
+			
+			/* literal */
+			if (camel_imap4_engine_next_token (engine, token, ex) == -1)
+				goto exception;
+			
+			if (token->token != CAMEL_IMAP4_TOKEN_LITERAL)
+				goto unexpected;
+			
+			parser = camel_mime_parser_new ();
+			camel_mime_parser_init_with_stream (parser, (CamelStream *) engine->istream);
+			
+			switch (camel_mime_parser_step (parser, NULL, NULL)) {
+			case CAMEL_MIME_PARSER_STATE_HEADER:
+			case CAMEL_MIME_PARSER_STATE_MESSAGE:
+			case CAMEL_MIME_PARSER_STATE_MULTIPART:
+				h = camel_mime_parser_headers_raw (parser);
+				iinfo->info.mlist = camel_header_raw_check_mailing_list (&h);
+			default:
+				break;
+			}
+			
+			camel_object_unref (parser);
 		} else {
 			/* wtf? */
 			d(fprintf (stderr, "huh? %s?...\n", token->v.atom));
@@ -973,6 +1058,7 @@ untagged_fetch_all (CamelIMAP4Engine *engine, CamelIMAP4Command *ic, guint32 ind
 }
 
 #define IMAP4_ALL "FLAGS INTERNALDATE RFC822.SIZE ENVELOPE"
+#define MAILING_LIST_HEADERS " BODY.PEEK[HEADER.FIELDS (List-Post List-Id Mailing-List Originator X-Mailing-List X-Loop X-List Sender Delivered-To Return-Path X-BeenThere List-Unsubscribe)]"
 
 static CamelIMAP4Command *
 imap4_summary_fetch_all (CamelFolderSummary *summary, guint32 first, guint32 last)
@@ -982,6 +1068,7 @@ imap4_summary_fetch_all (CamelFolderSummary *summary, guint32 first, guint32 las
 	struct imap4_fetch_all_t *fetch;
 	CamelIMAP4Engine *engine;
 	CamelIMAP4Command *ic;
+	const char *query;
 	int total;
 	
 	engine = ((CamelIMAP4Store *) folder->parent_store)->engine;
@@ -997,10 +1084,21 @@ imap4_summary_fetch_all (CamelFolderSummary *summary, guint32 first, guint32 las
 	fetch->total = total;
 	fetch->count = 0;
 	
-	if (last != 0)
-		ic = camel_imap4_engine_queue (engine, folder, "FETCH %u:%u (UID " IMAP4_ALL ")\r\n", first, last);
-	else
-		ic = camel_imap4_engine_queue (engine, folder, "FETCH %u:* (UID " IMAP4_ALL ")\r\n", first);
+	if (last != 0) {
+		if (((CamelIMAP4Folder *) folder)->enable_mlist)
+			query = "FETCH %u:%u (UID " IMAP4_ALL MAILING_LIST_HEADERS ")\r\n";
+		else
+			query = "FETCH %u:%u (UID " IMAP4_ALL ")\r\n";
+		
+		ic = camel_imap4_engine_queue (engine, folder, query, first, last);
+	} else {
+		if (((CamelIMAP4Folder *) folder)->enable_mlist)
+			query = "FETCH %u:* (UID " IMAP4_ALL MAILING_LIST_HEADERS ")\r\n";
+		else
+			query = "FETCH %u:* (UID " IMAP4_ALL ")\r\n";
+		
+		ic = camel_imap4_engine_queue (engine, folder, query, first);
+	}
 	
 	camel_imap4_command_register_untagged (ic, "FETCH", untagged_fetch_all);
 	ic->user_data = fetch;
@@ -1123,6 +1221,40 @@ imap4_message_info_save (CamelFolderSummary *summary, FILE *fout, CamelMessageIn
 	return 0;
 }
 
+static CamelMessageInfo *
+imap4_message_info_clone (CamelFolderSummary *summary, const CamelMessageInfo *mi)
+{
+	const CamelIMAP4MessageInfo *src = (const CamelIMAP4MessageInfo *) mi;
+	CamelIMAP4MessageInfo *dest;
+	
+	dest = (CamelIMAP4MessageInfo *) CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_clone (summary, mi);
+	dest->server_flags = src->server_flags;
+	
+	/* FIXME: parent clone should do this */
+	dest->info.content = camel_folder_summary_content_info_new (summary);
+	
+	return (CamelMessageInfo *) dest;
+}
+
+static CamelMessageContentInfo *
+imap4_content_info_load (CamelFolderSummary *summary, FILE *in)
+{
+	if (fgetc (in))
+		return CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->content_info_load (summary, in);
+	else
+		return camel_folder_summary_content_info_new (summary);
+}
+
+static int
+imap4_content_info_save (CamelFolderSummary *summary, FILE *out, CamelMessageContentInfo *info)
+{
+	if (info->type) {
+		fputc (1, out);
+		return CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->content_info_save (summary, out, info);
+	} else
+		return fputc (0, out);
+}
+
 
 void
 camel_imap4_summary_set_exists (CamelFolderSummary *summary, guint32 exists)
@@ -1245,6 +1377,7 @@ info_uid_sort (const CamelMessageInfo **info0, const CamelMessageInfo **info1)
 int
 camel_imap4_summary_flush_updates (CamelFolderSummary *summary, CamelException *ex)
 {
+	CamelOfflineJournal *journal = ((CamelIMAP4Folder *) summary->folder)->journal;
 	CamelIMAP4Summary *imap4_summary = (CamelIMAP4Summary *) summary;
 	CamelIMAP4Engine *engine;
 	CamelIMAP4Command *ic;
@@ -1254,7 +1387,7 @@ camel_imap4_summary_flush_updates (CamelFolderSummary *summary, CamelException *
 	g_return_val_if_fail (CAMEL_IS_IMAP4_SUMMARY (summary), -1);
 	
 	/* FIXME: what do we do if replaying the journal fails? */
-	camel_offline_journal_replay (((CamelIMAP4Folder *) summary->folder)->journal, NULL);
+	camel_offline_journal_replay (journal, NULL);
 	
 	engine = ((CamelIMAP4Store *) summary->folder->parent_store)->engine;
 	scount = camel_folder_summary_count (summary);
@@ -1272,6 +1405,7 @@ camel_imap4_summary_flush_updates (CamelFolderSummary *summary, CamelException *
 			;
 		
 		if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
+			camel_imap4_journal_readd_failed ((CamelIMAP4Journal *) journal);
 			imap4_fetch_all_free (ic->user_data);
 			camel_exception_xfer (ex, &ic->ex);
 			camel_imap4_command_unref (ic);
@@ -1296,6 +1430,7 @@ camel_imap4_summary_flush_updates (CamelFolderSummary *summary, CamelException *
 			;
 		
 		if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
+			camel_imap4_journal_readd_failed ((CamelIMAP4Journal *) journal);
 			imap4_fetch_all_free (ic->user_data);
 			camel_exception_xfer (ex, &ic->ex);
 			camel_imap4_command_unref (ic);
@@ -1319,5 +1454,7 @@ camel_imap4_summary_flush_updates (CamelFolderSummary *summary, CamelException *
 	imap4_summary->update_flags = FALSE;
 	imap4_summary->uidvalidity_changed = FALSE;
 	
+	camel_imap4_journal_readd_failed ((CamelIMAP4Journal *) journal);
+	
 	return 0;
 }
