@@ -44,15 +44,22 @@ typedef struct {
 }
 Section;
 
-static ESource *find_first_source         (ESourceList *source_list);
-static void     search_changed            (ENameSelectorDialog *name_selector_dialog);
-static void     source_selected           (ENameSelectorDialog *name_selector_dialog, ESource *source);
-static void     transfer_button_clicked   (ENameSelectorDialog *name_selector_dialog, GtkButton *transfer_button);
-static void     contact_selection_changed (ENameSelectorDialog *name_selector_dialog);
-static void     setup_name_selector_model (ENameSelectorDialog *name_selector_dialog);
-static void     contact_activated         (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path);
-static void     destination_activated     (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path,
-					   GtkTreeViewColumn *column, GtkTreeView *tree_view);
+static ESource *find_first_source             (ESourceList *source_list);
+static void     search_changed                (ENameSelectorDialog *name_selector_dialog);
+static void     source_selected               (ENameSelectorDialog *name_selector_dialog, ESource *source);
+static void     transfer_button_clicked       (ENameSelectorDialog *name_selector_dialog, GtkButton *transfer_button);
+static void     contact_selection_changed     (ENameSelectorDialog *name_selector_dialog);
+static void     setup_name_selector_model     (ENameSelectorDialog *name_selector_dialog);
+static void     contact_activated             (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path);
+static void     destination_activated         (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path,
+					       GtkTreeViewColumn *column, GtkTreeView *tree_view);
+static void     remove_books                  (ENameSelectorDialog *name_selector_dialog);
+static void     contact_column_formatter      (GtkTreeViewColumn *column, GtkCellRenderer *cell,
+					       GtkTreeModel *model, GtkTreeIter *iter,
+					       ENameSelectorDialog *name_selector_dialog);
+static void     destination_column_formatter  (GtkTreeViewColumn *column, GtkCellRenderer *cell,
+					       GtkTreeModel *model, GtkTreeIter *iter,
+					       ENameSelectorDialog *name_selector_dialog);
 
 /* ------------------ *
  * Class/object setup *
@@ -129,7 +136,9 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	column = gtk_tree_view_column_new ();
 	cell_renderer = GTK_CELL_RENDERER (gtk_cell_renderer_text_new ());
 	gtk_tree_view_column_pack_start (column, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute (column, cell_renderer, "text", E_CONTACT_FILE_AS);
+	gtk_tree_view_column_set_cell_data_func (column, cell_renderer,
+						 (GtkTreeCellDataFunc) contact_column_formatter,
+						 name_selector_dialog, NULL);
 	gtk_tree_view_append_column (name_selector_dialog->contact_view, column);
 	g_signal_connect_swapped (name_selector_dialog->contact_view, "row-activated",
 				  G_CALLBACK (contact_activated), name_selector_dialog);
@@ -189,6 +198,12 @@ e_name_selector_dialog_dispose (GObject *object)
 {
 	ENameSelectorDialog *name_selector_dialog = E_NAME_SELECTOR_DIALOG (object);
 
+	g_signal_handlers_disconnect_matched (name_selector_dialog->name_selector_model,
+					      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL,
+					      name_selector_dialog);
+
+	remove_books (name_selector_dialog);
+
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose)
 		G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose (object);
 }
@@ -231,17 +246,37 @@ e_name_selector_dialog_new (void)
  * Utilities *
  * --------- */
 
-static void
-sort_iter_to_contact_store_iter (ENameSelectorDialog *name_selector_dialog, GtkTreeIter *iter)
+static gchar *
+escape_sexp_string (const gchar *string)
 {
-	GtkTreeModelFilter *contact_filter;
-	GtkTreeIter         child_iter;
+	GString *gstring;
+	gchar   *encoded_string;
+
+	gstring = g_string_new ("");
+	e_sexp_encode_string (gstring, string);
+
+	encoded_string = gstring->str;
+	g_string_free (gstring, FALSE);
+
+	return encoded_string;
+}
+
+static void
+sort_iter_to_contact_store_iter (ENameSelectorDialog *name_selector_dialog, GtkTreeIter *iter,
+				 gint *email_n)
+{
+	ETreeModelGenerator *contact_filter;
+	GtkTreeIter          child_iter;
+	gint                 email_n_local;
 
 	contact_filter = e_name_selector_model_peek_contact_filter (name_selector_dialog->name_selector_model);
 
 	gtk_tree_model_sort_convert_iter_to_child_iter (name_selector_dialog->contact_sort,
 							&child_iter, iter);
-	gtk_tree_model_filter_convert_iter_to_child_iter (contact_filter, iter, &child_iter);
+	e_tree_model_generator_convert_iter_to_child_iter (contact_filter, iter, &email_n_local, &child_iter);
+
+	if (email_n)
+		*email_n = email_n_local;
 }
 
 static ESource *
@@ -265,7 +300,7 @@ find_first_source (ESourceList *source_list)
 }
 
 static void
-add_destination (EDestinationStore *destination_store, EContact *contact)
+add_destination (EDestinationStore *destination_store, EContact *contact, gint email_n)
 {
 	EDestination *destination;
 
@@ -273,11 +308,36 @@ add_destination (EDestinationStore *destination_store, EContact *contact)
 	 * source automatically) */
 
 	destination = e_destination_new ();
-	e_destination_set_contact (destination, contact, 0);
+	e_destination_set_contact (destination, contact, email_n);
 	e_destination_store_append_destination (destination_store, destination);
 	g_object_unref (destination);
 }
 
+static void
+remove_books (ENameSelectorDialog *name_selector_dialog)
+{
+	EContactStore *contact_store;
+	GList         *books;
+	GList         *l;
+
+	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
+
+	/* Remove books (should be just one) being viewed */
+	books = e_contact_store_get_books (contact_store);
+	for (l = books; l; l = g_list_next (l)) {
+		EBook *book = l->data;
+		e_contact_store_remove_book (contact_store, book);
+	}
+	g_list_free (books);
+
+	/* See if we have a book pending; stop loading it if so */
+	if (name_selector_dialog->pending_book) {
+		e_book_cancel (name_selector_dialog->pending_book, NULL);
+		g_object_unref (name_selector_dialog->pending_book);
+		name_selector_dialog->pending_book = NULL;
+	}
+}
+
 /* ------------------ *
  * Section management *
  * ------------------ */
@@ -355,8 +415,9 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 	column = gtk_tree_view_column_new ();
 	cell_renderer = GTK_CELL_RENDERER (gtk_cell_renderer_text_new ());
 	gtk_tree_view_column_pack_start (column, cell_renderer, TRUE);
-	gtk_tree_view_column_add_attribute (column, cell_renderer,
-					    "text", E_DESTINATION_STORE_COLUMN_ADDRESS);
+	gtk_tree_view_column_set_cell_data_func (column, cell_renderer,
+						 (GtkTreeCellDataFunc) destination_column_formatter,
+						 name_selector_dialog, NULL);
 	gtk_tree_view_append_column (section.destination_view, column);
 	gtk_tree_view_set_headers_visible (section.destination_view, FALSE);
 	gtk_tree_view_set_model (section.destination_view, GTK_TREE_MODEL (destination_store));
@@ -458,25 +519,14 @@ source_selected (ENameSelectorDialog *name_selector_dialog, ESource *source)
 	GList         *books;
 	GList         *l;
 
-	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
-
-	/* Remove books (should be just one) being viewed */
-	books = e_contact_store_get_books (contact_store);
-	for (l = books; l; l = g_list_next (l)) {
-		EBook *book = l->data;
-		e_contact_store_remove_book (contact_store, book);
-	}
-	g_list_free (books);
-
-	/* See if we already have a book pending; stop loading it if so */
-	if (name_selector_dialog->pending_book) {
-		e_book_cancel (name_selector_dialog->pending_book, NULL);
-		g_object_unref (name_selector_dialog->pending_book);
-		name_selector_dialog->pending_book = NULL;
-	}
+	/* Remove any previous books being shown or loaded */
+	remove_books (name_selector_dialog);
 
 	/* Start loading selected book */
 	name_selector_dialog->pending_book = e_book_new (source, NULL);
+	if (!name_selector_dialog->pending_book)
+		return;
+
 	e_book_async_open (name_selector_dialog->pending_book, TRUE,
 			   book_opened, name_selector_dialog);
 }
@@ -491,12 +541,15 @@ search_changed (ENameSelectorDialog *name_selector_dialog)
 	EContactStore *contact_store;
 	EBookQuery    *book_query;
 	const gchar   *text;
+	gchar         *text_escaped;
 	gchar         *query_string;
 
 	text = gtk_entry_get_text (name_selector_dialog->search_entry);
-	query_string = g_strdup_printf ("(contains \"file_as\" \"%s\")", text);
+	text_escaped = escape_sexp_string (text);
+	query_string = g_strdup_printf ("(contains \"file_as\" %s)", text_escaped);
 	book_query = e_book_query_from_string (query_string);
 	g_free (query_string);
+	g_free (text_escaped);
 
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 	e_contact_store_set_query (contact_store, book_query);
@@ -528,6 +581,7 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 	EContact          *contact;
 	GtkTreeIter        iter;
 	Section           *section;
+	gint               email_n;
 
 	/* When a contact is activated, we transfer it to the first destination on our list */
 
@@ -544,7 +598,7 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 				      &iter, path))
 		g_assert_not_reached ();
 
-	sort_iter_to_contact_store_iter (name_selector_dialog, &iter);
+	sort_iter_to_contact_store_iter (name_selector_dialog, &iter, &email_n);
 
 	contact = e_contact_store_get_contact (contact_store, &iter);
 	if (!contact) {
@@ -559,7 +613,7 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 		return;
 	}
 
-	add_destination (destination_store, contact);
+	add_destination (destination_store, contact, email_n);
 }
 
 static void
@@ -607,6 +661,7 @@ transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *t
 	gint               section_index;
 	Section           *section;
 	EDestination      *destination;
+	gint               email_n;
 
 	/* Get the contact to be transferred */
 
@@ -618,7 +673,7 @@ transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *t
 		return;
 	}
 
-	sort_iter_to_contact_store_iter (name_selector_dialog, &iter);
+	sort_iter_to_contact_store_iter (name_selector_dialog, &iter, &email_n);
 
 	contact = e_contact_store_get_contact (contact_store, &iter);
 	if (!contact) {
@@ -641,7 +696,7 @@ transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *t
 		return;
 	}
 
-	add_destination (destination_store, contact);
+	add_destination (destination_store, contact, email_n);
 }
 
 /* --------------------- *
@@ -651,13 +706,13 @@ transfer_button_clicked (ENameSelectorDialog *name_selector_dialog, GtkButton *t
 static void
 setup_name_selector_model (ENameSelectorDialog *name_selector_dialog)
 {
-	EContactStore      *contact_store;
-	GtkTreeModelFilter *contact_filter;
-	GtkTreeSelection   *contact_selection;
-	EBookQuery         *book_query;
-	GList              *new_sections;
-	GList              *l;
-	gint                i;
+	EContactStore       *contact_store;
+	ETreeModelGenerator *contact_filter;
+	GtkTreeSelection    *contact_selection;
+	EBookQuery          *book_query;
+	GList               *new_sections;
+	GList               *l;
+	gint                 i;
 
 	/* Rid UI of previous destination sections */
 
@@ -716,6 +771,86 @@ setup_name_selector_model (ENameSelectorDialog *name_selector_dialog)
 	contact_selection_changed (name_selector_dialog);
 }
 
+static void
+deep_free_list (GList *list)
+{
+	GList *l;
+
+	for (l = list; l; l = g_list_next (l))
+		g_free (l->data);
+
+	g_list_free (list);
+}
+
+static void
+contact_column_formatter (GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *model,
+			  GtkTreeIter *iter, ENameSelectorDialog *name_selector_dialog)
+{
+	EContactStore *contact_store;
+	EContact      *contact;
+	GtkTreeIter    contact_store_iter;
+	GList         *email_list;
+	gchar         *string;
+	gchar         *file_as_str;
+	gchar         *email_str;
+	gint           email_n;
+
+	contact_store_iter = *iter;
+	sort_iter_to_contact_store_iter (name_selector_dialog, &contact_store_iter, &email_n);
+
+	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
+	contact = e_contact_store_get_contact (contact_store, &contact_store_iter);
+	email_list = e_contact_get (contact, E_CONTACT_EMAIL);
+	email_str = g_list_nth_data (email_list, email_n);
+	file_as_str = e_contact_get (contact, E_CONTACT_FILE_AS);
+
+	if (e_contact_get (contact, E_CONTACT_IS_LIST)) {
+		string = g_strdup_printf ("%s", file_as_str ? file_as_str : "?");
+	} else {
+		string = g_strdup_printf ("%s%s<%s>", file_as_str ? file_as_str : "",
+					  file_as_str ? " " : "",
+					  email_str ? email_str : "");
+	}
+
+	g_free (file_as_str);
+	deep_free_list (email_list);
+
+	g_object_set (cell, "text", string, NULL);
+	g_free (string);
+}
+
+static void
+destination_column_formatter (GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *model,
+			      GtkTreeIter *iter, ENameSelectorDialog *name_selector_dialog)
+{
+	EDestinationStore *destination_store = E_DESTINATION_STORE (model);
+	EDestination      *destination;
+	gchar             *string;
+
+       	destination = e_destination_store_get_destination (destination_store, iter);
+	g_assert (destination);
+
+	if (e_destination_is_evolution_list (destination)) {
+		if (e_destination_list_show_addresses (destination)) {
+			const gchar *name;
+			const gchar *addresses;
+
+			name      = e_destination_get_name (destination);
+			addresses = e_destination_get_address (destination);
+
+			string = g_strdup_printf ("%s%s(%s)", name ? name : "",
+						  name ? " " : "", addresses ? addresses : "?");
+		} else {
+			string = g_strdup (e_destination_get_name (destination));
+		}
+	} else {
+		string = g_strdup (e_destination_get_address (destination));
+	}
+
+	g_object_set (cell, "text", string, NULL);
+	g_free (string);
+}
+
 /* ----------------------- *
  * ENameSelectorDialog API *
  * ----------------------- */
