@@ -3,7 +3,7 @@
  * Authors : 
  *  JP Rosevear <jpr@ximian.com>
  *  Rodrigo Moya <rodrigo@ximian.com>
- *
+ *  Harish Krishnaswamy <kharish@novell.com>
  * Copyright 2003, Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or 
@@ -52,6 +52,7 @@ struct _EGwItemPrivate {
 	char *priority;
 	char *task_priority;
 	char *place;
+	char *source ;
 	GSList *recipient_list;
 	GSList *recurrence_dates;
 	int trigger; /* alarm */
@@ -85,6 +86,7 @@ struct _EGwItemPrivate {
 
 	/***** Send Options *****/
 	
+	gboolean set_sendoptions;
 	/* Reply Request */
 	char *reply_within;
 	gboolean reply_request_set;
@@ -120,6 +122,13 @@ free_recipient (EGwItemRecipient *recipient, gpointer data)
 	g_free (recipient->email);
 	g_free (recipient->display_name);
 	g_free (recipient);
+	g_free (recipient->delivered_date);
+	g_free (recipient->opened_date);
+	g_free (recipient->accepted_date);
+	g_free (recipient->deleted_date);
+	g_free (recipient->declined_date);
+	g_free (recipient->completed_date);
+	g_free (recipient->undelivered_date);
 }
 
 static void 
@@ -171,6 +180,8 @@ free_attach (gpointer s, gpointer data)
 			g_free (attach->contentType), attach->contentType = NULL ;
 		if (attach->date)
 			g_free (attach->date), attach->date = NULL ;
+		if (attach->data)
+			g_free (attach->data), attach->data = NULL ;
 	
 		g_free(attach) ;
 	}
@@ -323,7 +334,8 @@ e_gw_item_dispose (GObject *object)
 			priv->recipient_list = NULL;
 		}	
 		if (priv->organizer) {
-			g_free (priv->organizer);
+			g_free (priv->organizer->display_name);
+			g_free (priv->organizer->email);
 			priv->organizer = NULL;
 		}
 		if (priv->recurrence_dates) {
@@ -369,6 +381,10 @@ e_gw_item_dispose (GObject *object)
 			g_free (priv->category_name);
 			priv->category_name = NULL;
 		}
+		if (priv->source) {
+			g_free (priv->source);
+			priv->source = NULL;
+		}
 		free_changes (priv->additions);
 		free_changes (priv->deletions);
 		free_changes (priv->updates);
@@ -434,6 +450,7 @@ e_gw_item_init (EGwItem *item, EGwItemClass *klass)
 	priv->reply_within = NULL;
 	priv->reply_request_set = FALSE;
 	priv->autodelete = FALSE;
+	priv->set_sendoptions = FALSE;
 	priv->expires = NULL;
 	priv->delay_until = NULL;
 	priv->attach_list = NULL ;
@@ -527,22 +544,64 @@ set_recipient_list_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 		 to none. */
 		subparam = soup_soap_parameter_get_first_child_by_name (param_recipient, "recipientStatus");
                 if (subparam) {
-                        if (soup_soap_parameter_get_first_child_by_name (subparam, "deleted"))
+       			char *formatted_date, *value; 
+			SoupSoapParameter *temp_param ;
+
+			recipient->status_enabled = TRUE;
+       			if ( (temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "deleted")) ) {
 				recipient->status = E_GW_ITEM_STAT_DECLINED;
-			if (soup_soap_parameter_get_first_child_by_name (subparam, "declined"))
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				recipient->deleted_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
+			} 
+			if ( (temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "declined")) ) {
 				recipient->status = E_GW_ITEM_STAT_DECLINED;
-			else if (soup_soap_parameter_get_first_child_by_name (subparam, "accepted")) 
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				recipient->declined_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
+
+			} if ( (temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "accepted"))) {
 				recipient->status = E_GW_ITEM_STAT_ACCEPTED;
-			else 	
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				recipient->accepted_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
+			} else 	
 				recipient->status = E_GW_ITEM_STAT_NONE;
-			subparam = soup_soap_parameter_get_first_child_by_name (subparam, "completed");
-			if (subparam) {
-				char *formatted_date, *value; 
-				value = soup_soap_parameter_get_string_value (subparam);
+			temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "completed");
+			if (temp_param) {
+				value = soup_soap_parameter_get_string_value (temp_param);
 				formatted_date = e_gw_connection_format_date_string (value);
 				e_gw_item_set_completed_date (item, formatted_date);
-				g_free (value);
-				g_free (formatted_date);
+				recipient->completed_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
+			}
+			if ( (temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "opened"))) {
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				recipient->opened_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
+			}
+			if ( ( temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "delivered"))) {
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				recipient->delivered_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
+			}
+			if ( (temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "undeliverable_date"))) {
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				recipient->undelivered_date = g_strdup (formatted_date);
+				g_free (value), value = NULL;
+				g_free (formatted_date), formatted_date = NULL;
 			}
                 }
 		else {
@@ -1377,7 +1436,7 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 {
 	EGwItem *item;
         char *item_type;
-	SoupSoapParameter *subparam, *child, *category_param, *attachment_param;
+	SoupSoapParameter *subparam, *child, *category_param, *attachment_param, *notification_param ;
 	gboolean is_group_item = TRUE;
 	GList *user_email = NULL;
 	
@@ -1401,6 +1460,9 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 		set_contact_fields_from_soap_parameter (item, param);
 		return item;
 	} 
+	else if (!g_ascii_strcasecmp (item_type,"SharedNotification"))
+		item->priv->item_type = E_GW_ITEM_TYPE_NOTIFICATION ;
+
 	else if (!g_ascii_strcasecmp (item_type, "Organization")) {
 
 		item->priv->item_type =  E_GW_ITEM_TYPE_ORGANISATION;
@@ -1606,7 +1668,7 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 				EGwItemAttachment *attach = g_new0 (EGwItemAttachment, 1) ;
 				temp = soup_soap_parameter_get_first_child_by_name (attachment_param, "id") ;
 				if (temp)	
-					attach->id = soup_soap_parameter_get_string_value (temp), printf ("||| attach id:%s |||\n",attach->id) ;
+					attach->id = soup_soap_parameter_get_string_value (temp) ;
 				
 				temp = soup_soap_parameter_get_first_child_by_name (attachment_param, "name") ;
 				if (temp)
@@ -2010,6 +2072,17 @@ e_gw_item_get_organizer (EGwItem *item)
 	return item->priv->organizer;
 }
 
+void
+e_gw_item_set_attach_id_list (EGwItem *item, GSList *attach_list)
+{
+	g_return_if_fail (E_IS_GW_ITEM (item)) ;
+	if (attach_list) {
+		g_slist_foreach (item->priv->attach_list, (GFunc)free_attach, NULL) ;
+		g_slist_free (item->priv->attach_list) ;
+	}
+	item->priv->attach_list = attach_list ;
+}
+
 GSList *
 e_gw_item_get_attach_id_list (EGwItem *item)
 {
@@ -2080,6 +2153,14 @@ e_gw_item_get_msg_content_type (EGwItem *item)
 	return item->priv->content_type ;
 }
 
+void
+e_gw_item_set_sendoptions (EGwItem *item, gboolean set)
+{
+	g_return_if_fail (E_IS_GW_ITEM (item));
+
+	item->priv->set_sendoptions = set;
+}
+
 void
 e_gw_item_set_reply_request (EGwItem *item, gboolean set)
 {
@@ -2257,6 +2338,13 @@ e_gw_item_get_delay_until (EGwItem *item)
 	return item->priv->delay_until;
 }
 
+void
+e_gw_item_set_source (EGwItem *item, char *source)
+{
+	g_return_if_fail (E_IS_GW_ITEM (item));
+	item->priv->source = g_strdup (source);
+}
+
 static void
 add_return_notification (SoupSoapMessage *msg, char *option, EGwItemReturnNotify value)
 {
@@ -2350,55 +2438,94 @@ add_distribution_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 	
 	soup_soap_message_end_element (msg);
 	
-	soup_soap_message_start_element (msg, "sendoptions", NULL, NULL);
+	if (priv->set_sendoptions) {	
+		soup_soap_message_start_element (msg, "sendoptions", NULL, NULL);
+		
+		if (priv->reply_request_set) {
+			
+			soup_soap_message_start_element (msg, "requestReply", NULL, NULL);
+			
+			if (priv->reply_within)
+				e_gw_message_write_string_parameter (msg, "withinNDays", NULL, priv->reply_within);
+			
+			soup_soap_message_end_element (msg);
+		}
 	
-	if (priv->reply_request_set) {
+		soup_soap_message_start_element (msg, "statusTracking", NULL, NULL);
 		
-		soup_soap_message_start_element (msg, "requestReply", NULL, NULL);
+		soup_soap_message_add_attribute (msg, "autoDelete", priv->autodelete ? "1" : "0", NULL, NULL);
+	
+		switch (priv->track_info) {
+			case E_GW_ITEM_DELIVERED : soup_soap_message_write_string (msg, "Delivered");
+				 break;
+			case E_GW_ITEM_DELIVERED_OPENED : soup_soap_message_write_string (msg, "DeliveredAndOpened");
+				 break;
+			case E_GW_ITEM_ALL : soup_soap_message_write_string (msg, "All");
+				 break;
+			default: soup_soap_message_write_string (msg, "None");
+		}
 		
-		if (priv->reply_within)
-			e_gw_message_write_string_parameter (msg, "withinNDays", NULL, priv->reply_within);
+		soup_soap_message_end_element (msg);
+	
+		soup_soap_message_start_element (msg, "notification", NULL, NULL);
+		switch (priv->item_type) {
 		
+		/* TODO Uncomment this after the completed element is available in shemas */
+		case E_GW_ITEM_TYPE_TASK :
+	//		add_return_notification (msg, "completed", priv->notify_completed);
+			
+		case E_GW_ITEM_TYPE_APPOINTMENT:
+			add_return_notification (msg, "accepted", priv->notify_accepted);
+			add_return_notification (msg, "declined", priv->notify_declined);
+			add_return_notification (msg, "opened", priv->notify_opened);
+			break;
+		
+		default:
+			add_return_notification (msg, "opened", priv->notify_opened);
+			add_return_notification (msg, "deleted", priv->notify_deleted);
+		}
+		soup_soap_message_end_element (msg);
+
 		soup_soap_message_end_element (msg);
 	}
 
-	soup_soap_message_start_element (msg, "statusTracking", NULL, NULL);
-	
-	soup_soap_message_add_attribute (msg, "autoDelete", priv->autodelete ? "1" : "0", NULL, NULL);
+	soup_soap_message_end_element (msg);
+}
 
-	switch (priv->track_info) {
-		case E_GW_ITEM_DELIVERED : soup_soap_message_write_string (msg, "Delivered");
-			 break;
-		case E_GW_ITEM_DELIVERED_OPENED : soup_soap_message_write_string (msg, "DeliveredAndOpened");
-			 break;
-		case E_GW_ITEM_ALL : soup_soap_message_write_string (msg, "All");
-			 break;
-		default: soup_soap_message_write_string (msg, "None");
-	}
-	
-	soup_soap_message_end_element (msg);
+static void
+add_attachment_to_soap_message(EGwItemAttachment *attachment, SoupSoapMessage *msg)
+{
+	char *size ;
+	soup_soap_message_start_element (msg, "attachment", NULL, NULL) ;
 
-	soup_soap_message_start_element (msg, "notification", NULL, NULL);
-	switch (priv->item_type) {
-	
-	/* TODO Uncomment this after the completed element is available in shemas */
-	case E_GW_ITEM_TYPE_TASK :
-//		add_return_notification (msg, "completed", priv->notify_completed);
-		
-	case E_GW_ITEM_TYPE_APPOINTMENT:
-		add_return_notification (msg, "accepted", priv->notify_accepted);
-		add_return_notification (msg, "declined", priv->notify_declined);
-		add_return_notification (msg, "opened", priv->notify_opened);
-		break;
-		
-	default:
-		add_return_notification (msg, "opened", priv->notify_opened);
-		add_return_notification (msg, "deleted", priv->notify_deleted);
-	}
-	soup_soap_message_end_element (msg);
+	/*id*/
+	if (attachment->id)
+		e_gw_message_write_string_parameter (msg, "id", NULL, attachment->id) ;
+	else
+		e_gw_message_write_string_parameter (msg, "id", NULL, "") ;
+	/*name*/
+	e_gw_message_write_string_parameter (msg, "name", NULL, attachment->name) ;
+	/*content type*/
+	e_gw_message_write_string_parameter (msg, "contentType", NULL, attachment->contentType) ;
+	/*size*/
+	size = g_strdup_printf ("%d", attachment->size) ;
+	e_gw_message_write_string_parameter (msg, "size", NULL, size) ;
+	/*date*/
+	if (attachment->date) 
+		e_gw_message_write_string_parameter (msg, "date", NULL, attachment->date) ;	
+	else
+		e_gw_message_write_string_parameter (msg, "date", NULL, "") ;	
 
-	soup_soap_message_end_element (msg);
-	soup_soap_message_end_element (msg);
+	/*data*/
+	soup_soap_message_start_element (msg, "data", NULL, NULL) ;
+	soup_soap_message_add_attribute (msg, "contentId", attachment->id, NULL, NULL);
+	soup_soap_message_add_attribute (msg, "contentType", attachment->contentType, NULL, NULL) ;
+	soup_soap_message_add_attribute (msg, "length", size, NULL, NULL) ;
+	soup_soap_message_write_string (msg, attachment->data) ;
+	soup_soap_message_end_element (msg) ;
+	
+	soup_soap_message_end_element (msg) ;
+	g_free (size) ;
 }
 
 static void 
@@ -2421,7 +2548,8 @@ e_gw_item_set_calendar_item_elements (EGwItem *item, SoupSoapMessage *msg)
 
 	if (priv->recipient_list != NULL) {
 		add_distribution_to_soap_message (item, msg);
-		append_gw_item_options (msg, item);
+		if (priv->set_sendoptions)
+			append_gw_item_options (msg, item);
 	}
 
 	soup_soap_message_start_element (msg, "message", NULL, NULL);
@@ -2429,7 +2557,7 @@ e_gw_item_set_calendar_item_elements (EGwItem *item, SoupSoapMessage *msg)
 		char *str;
 		
 		str = soup_base64_encode (priv->message, strlen (priv->message));
-		dtstring = g_strdup_printf ("%d", strlen (str));
+		dtstring = g_strdup_printf ("%d", (int)strlen (str));
 		soup_soap_message_add_attribute (msg, "length", dtstring, NULL, NULL);
 		g_free (dtstring);
 		soup_soap_message_write_string (msg, str);
@@ -2473,6 +2601,18 @@ e_gw_item_set_calendar_item_elements (EGwItem *item, SoupSoapMessage *msg)
 		 * recurring items */
 		e_gw_message_write_string_parameter (msg, "iCalId", NULL, priv->icalid ? priv->icalid : "");
 	}
+
+	/*attachments*/
+	if (priv->attach_list) {
+		soup_soap_message_start_element (msg, "attachments", NULL, NULL) ;
+		GSList *al ;
+		for (al = priv->attach_list ; al != NULL ;  al = al->next) {
+			EGwItemAttachment *attachment = (EGwItemAttachment *)al->data ;
+			add_attachment_to_soap_message (attachment, msg) ;
+		
+		}
+		soup_soap_message_end_element (msg) ;
+	}
 }
 
 gboolean
@@ -2498,8 +2638,15 @@ e_gw_item_append_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 		/*distribution*/
 		add_distribution_to_soap_message(item, msg) ;
 		
-		/* item options */
-		append_gw_item_options (msg, item);
+		if (priv->set_sendoptions) {
+			/* item options */
+			append_gw_item_options (msg, item);
+		}
+		/*container*/
+		if (item->priv->container)
+			e_gw_message_write_string_parameter (msg, "container", NULL, item->priv->container);
+		if (item->priv->source)
+			e_gw_message_write_string_parameter (msg, "source", NULL, item->priv->source );
 		
 		/*message*/
 		soup_soap_message_start_element (msg, "message", NULL, NULL);
@@ -2507,13 +2654,11 @@ e_gw_item_append_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 			char *str ;
 			char *str_len ;
 
-			printf ("||||| MESSAGE |||||\n%s\n",priv->message) ;
 			str = soup_base64_encode (priv->message, strlen (priv->message));
 			str_len = g_strdup_printf ("%d", strlen (str));
 			soup_soap_message_add_attribute (msg, "length", str_len, NULL, NULL);
 			g_free (str_len);
 			soup_soap_message_write_string (msg, str);
-			printf ("||||| ENCODED MESSAGE|||||\n%s\n",str) ;
 			g_free (str);
 		} else {
 			soup_soap_message_add_attribute (msg, "length", "0", NULL, NULL);
@@ -2522,6 +2667,17 @@ e_gw_item_append_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 
 		soup_soap_message_end_element (msg);
 
+		/*attachments*/
+		if (priv->attach_list) {
+			soup_soap_message_start_element (msg, "attachments", NULL, NULL) ;
+			GSList *al ;
+			for (al = priv->attach_list ; al != NULL ;  al = al->next) {
+				EGwItemAttachment *attachment = (EGwItemAttachment *)al->data ;
+				add_attachment_to_soap_message (attachment, msg) ;
+			}
+			soup_soap_message_end_element (msg) ;
+		}
+
 		break;
 
 	case E_GW_ITEM_TYPE_APPOINTMENT :
@@ -2723,7 +2879,7 @@ append_event_changes_to_soap_message (EGwItem *item, SoupSoapMessage *msg, int c
 			char *str, *message;
 
 			str = soup_base64_encode (priv->message, strlen (priv->message));
-			message = g_strdup_printf ("%d", strlen (str));
+			message = g_strdup_printf ("%d", (int)strlen (str));
 			soup_soap_message_add_attribute (msg, "length", message, NULL, NULL);
 			g_free (message);
 			soup_soap_message_write_string (msg, str);
